#!/usr/bin/env python3
# Gufw 20.04.1 - https://costales.github.io/projects/gufw/
# Copyright (C) 2008-2019 Marcos Alvarez Costales https://costales.github.io
#
# Gufw is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Gufw is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Gufw; if not, see http://www.gnu.org/licenses for more
# information.

from gufw.controller import Controller 
from gufw.instance   import Instance
from gufw.view.gufw  import Gufw


if __name__ == '__main__':
    
    app_instance = Instance()
    
    controler = Controller()
    
    gufw = Gufw(controler.get_frontend())
    
    app_instance.exit_app()



# Gufw 20.04.1 - https://costales.github.io/projects/gufw/
# Copyright (C) 2008-2019 Marcos Alvarez Costales https://costales.github.io
#
# Gufw is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Gufw is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Gufw; if not, see http://www.gnu.org/licenses for more
# information.

import gi
import re
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk

import gettext
from gettext import gettext as _
gettext.textdomain('gufw')

DIRECTION2NUM = {'in': 0,
                 'out': 1,
                 'both': 1}
LOGGING2NUM = {'': 0,
               'log': 1,
               'log-all': 2}
PROTO2NUM = {'': 0,
             'tcp': 1,
             'udp': 2 }


class Update:
    def __init__(self, gufw, ufw_row, description, cmd, policy, direction, proto, from_ip, from_port, to_ip, to_port, iface, routed, logging):
        self.gufw = gufw
        self.ufw_row = str(ufw_row)
        self.rule_description = description
        self.rule_cmd         = cmd
        self.rule_policy      = policy
        self.rule_direction   = direction
        self.rule_proto       = proto
        self.rule_from_ip     = from_ip
        self.rule_from_port   = from_port
        self.rule_to_ip       = to_ip
        self.rule_to_port     = to_port
        self.rule_iface       = iface
        self.rule_routed      = routed
        self.rule_logging     = logging

        self.builder = Gtk.Builder()
        self.builder.set_translation_domain('gufw')
        self.builder.add_from_file('/usr/share/gufw/ui/update.ui')
        
        self._set_objects_name()
        self._set_initial_values()
        
        self.win_update.set_transient_for(gufw.winMain)
        self.builder.connect_signals(self)
        self.win_update.show_all()
        
        if self.gufw.frontend.get_policy('routed') == 'disabled':
            self.update_routed_img.set_visible(False)
            self.update_routed.set_visible(False)
        
        # Set sensitive values
        self.update_from_ip.set_text(self.rule_from_ip)
        self.update_from_port.set_text(self.rule_from_port)
        self.update_to_ip.set_text(self.rule_to_ip)
        self.update_to_port.set_text(self.rule_to_port)
        # Translators: About the network interfaces in the OS
        if self.update_iface.get_active_text() == _("All Interfaces"):
            self.update_routed.set_sensitive(False)
            # Translators: About the network interfaces in the OS
            self.update_routed.set_tooltip_text(_("You need to set an Interface"))
    
    def _set_objects_name(self):
        self.win_update        = self.builder.get_object('UpdateRule')
        self.update_rule_name  = self.builder.get_object('update_rule_name')
        self.update_policy     = self.builder.get_object('update_policy')
        self.update_direction  = self.builder.get_object('update_direction')
        self.update_iface      = self.builder.get_object('update_iface')
        self.update_routed     = self.builder.get_object('update_routed')
        self.update_routed_img = self.builder.get_object('update_routed_img')
        self.update_log        = self.builder.get_object('update_log')
        self.update_protocol   = self.builder.get_object('update_protocol')
        self.update_from_ip    = self.builder.get_object('update_from_ip')
        self.update_from_port  = self.builder.get_object('update_from_port')
        self.update_to_ip      = self.builder.get_object('update_to_ip')
        self.update_to_port    = self.builder.get_object('update_to_port')
        self.warning           = self.builder.get_object('lbl_preconfig_info')
        self.warning_box       = self.builder.get_object('warningbox')
        self.update_btn        = self.builder.get_object('btnUpdate')
    
    def _set_initial_values(self):
        iface_num = 0
        routed_num = 0
        i = 0
        # Translators: About the network interfaces in the OS
        self.update_iface.append_text(_("All Interfaces"))
        self.update_routed.append_text(_("Not Forward"))
        for ifaceName in self.gufw.frontend.get_net_interfaces():
            self.update_iface.append_text(ifaceName)
            self.update_routed.append_text(ifaceName)
            if ifaceName == self.rule_iface:
                iface_num = i + 1
            if ifaceName == self.rule_routed:
                routed_num = i + 1
            i += 1
        self.update_rule_name.set_text(self.rule_description)
        self.update_policy.set_active(self.gufw.POLICY2NUM[self.rule_policy])
        self.update_direction.set_active(DIRECTION2NUM[self.rule_direction])
        self.update_iface.set_active(iface_num)
        self.update_routed.set_active(routed_num)
        self.update_log.set_active(LOGGING2NUM[self.rule_logging])
        self.update_protocol.set_active(PROTO2NUM[self.rule_proto])
    
    def _set_from_port_sensitive(self, value=True):
        self.update_protocol.set_sensitive(value)
        self.update_from_ip.set_sensitive(value)
        self.update_to_ip.set_sensitive(value)
        self.update_to_port.set_sensitive(value)
    
    def _set_to_port_sensitive(self, value=True):
        self.update_protocol.set_sensitive(value)
        self.update_from_ip.set_sensitive(value)
        self.update_to_ip.set_sensitive(value)
        self.update_from_port.set_sensitive(value)
    
    def _set_update_btn_control(self):
        if (':' in self.update_to_port.get_text() or ':' in self.update_from_port.get_text()) and not self.update_protocol.get_active():
            self.update_btn.set_sensitive(False)
            self.update_btn.set_tooltip_text(_("Choose a TCP or UDP Protocol with a range of ports"))
        else:
            self.update_btn.set_sensitive(True)
            self.update_btn.set_tooltip_text('')
    
    def on_update_protocol_changed(self, widget, data=None):
        self._set_update_btn_control()
    
    def on_update_to_port_changed(self, widget, data=None):
        if '/' in self.update_to_port.get_text():
            self._set_to_port_sensitive(False)
        else:
            self._set_to_port_sensitive(True)
        
        self._set_update_btn_control()
    
    def on_update_from_port_changed(self, widget, data=None):
        if '/' in self.update_from_port.get_text():
            self._set_from_port_sensitive(False)
        else:
            self._set_from_port_sensitive(True)
        
        self._set_update_btn_control()
    
    def on_btnUpdateCancel_clicked(self, widget, data=None):
        self.win_update.destroy()
    
    def on_UpdateRule_delete_event(self, widget, data=None):
        self.win_update.destroy()
    
    def on_update_copy_from_IP_clicked(self, widget, data=None):
        self.update_from_ip.set_text(self.gufw.frontend.get_internal_ip())
    
    def on_update_copy_to_IP_clicked(self, widget, data=None):
        self.update_to_ip.set_text(self.gufw.frontend.get_internal_ip())
    
    def on_update_rule_name_icon_press(self, widget, data=None, data2=None):
        self.update_rule_name.set_text('')
    
    def on_update_from_ip_icon_press(self, widget, data=None, data2=None):
        self.update_from_ip.set_text('')
    
    def on_update_to_ip_icon_press(self, widget, data=None, data2=None):
        self.update_to_ip.set_text('')
    
    def on_update_from_port_icon_press(self, widget, data=None, data2=None):
        self.update_from_port.set_text('')
    
    def on_update_to_port_icon_press(self, widget, data=None, data2=None):
        self.update_to_port.set_text('')
    
    def on_update_iface_changed(self, widget, data=None, data2=None):
        # Translators: About the network interfaces in the OS
        if self.update_iface.get_active_text() != _("All Interfaces"):
            self.update_routed.set_sensitive(True)
            self.update_routed.set_tooltip_text(_("The IP/Port will be forward to this interface"))
        else:
            self.update_routed.set_sensitive(False)
            self.update_routed.set_tooltip_text(_("You need to set an Interface for forwarding to this another interface"))
        
        # Not allow same iface when is routed
        if self.gufw.frontend.get_policy('routed') != 'disabled':
            self.update_routed.remove_all()
            # Translators: About traffic
            self.update_routed.append_text(_("Not Forward"))
            for ifaceName in self.gufw.frontend.get_net_interfaces(self.update_iface.get_active_text()):
                self.update_routed.append_text(ifaceName)
            self.update_routed.set_active(0)
    
    def on_btnUpdate_clicked(self, widget, data=None):
        if not self.gufw.validate_rule(self.win_update, self.update_from_ip.get_text(), self.update_from_port.get_text(), self.update_to_ip.get_text(), self.update_to_port.get_text(), '', self.update_routed.get_active_text()):
            return
        
        new_description = self.update_rule_name.get_text()
        new_policy      = self.gufw.NUM2POLICY[self.update_policy.get_active()]
        new_direction   = self.gufw.NUM2DIRECTION[self.update_direction.get_active()]
        new_logging     = self.gufw.NUM2LOGGING[self.update_log.get_active()]
        
        new_proto = ''
        if self.update_protocol.get_sensitive():
            new_proto = self.gufw.NUM2PROTO[self.update_protocol.get_active()]
        
        new_iface = ''
        # Translators: About the network interfaces in the OS
        if self.update_iface.get_sensitive() and self.update_iface.get_active_text() != _("All Interfaces"):
            new_iface = self.update_iface.get_active_text()
        
        new_routed = ''
        # Translators: About traffic
        if self.update_routed.get_sensitive() and self.update_routed.get_active_text() != _("Not Forward"):
            new_routed = self.update_routed.get_active_text()
        
        new_from_ip = new_from_port = new_to_ip = new_to_port = ''
        if self.update_from_ip.get_sensitive():
            new_from_ip = self.update_from_ip.get_text()
        if self.update_from_port.get_sensitive():
            new_from_port = self.update_from_port.get_text()
        if self.update_to_ip.get_sensitive():
            new_to_ip = self.update_to_ip.get_text()
        if self.update_to_port.get_sensitive():
            new_to_port = self.update_to_port.get_text()
        
        if (self.rule_description == new_description and
            self.rule_policy      == new_policy      and
            self.rule_direction   == new_direction   and
            self.rule_proto       == new_proto       and
            self.rule_from_ip     == new_from_ip     and
            self.rule_from_port   == new_from_port   and
            self.rule_to_ip       == new_to_ip       and
            self.rule_to_port     == new_to_port     and
            self.rule_iface       == new_iface       and
            self.rule_routed      == new_routed      and
            self.rule_logging     == new_logging):
            self.warning_box.set_message_type(3)
            self.warning.set_text(_("No changes were made!"))
            return
        
        # Delete the same rules
        same_rules_rows = self._get_same_rules(self.rule_cmd)
        for same_row in same_rules_rows:
            cmd = self.gufw.frontend.delete_rule(same_row)
            self.gufw.add_to_log(_("Editing rule (Removing): ") + new_description + ' | ' + cmd[0] + ' > ' + cmd[1].replace('\n', ' | '))
        
        # Add new
        insert_row = ''
        cmd = self.gufw.frontend.add_rule(new_description, insert_row, new_policy, new_direction, new_iface, new_routed, new_logging, new_proto, new_from_ip, new_from_port, new_to_ip, new_to_port)

        self.gufw.add_to_log(_("Editing rule (Adding): ") + new_description + ' | ' + cmd[1] + ' > ' + cmd[2].replace('\n', ' | '), self.gufw.POLICY2COLOR[new_policy])
        self.gufw.set_statusbar_msg(_("Updated rule ") + str(self.ufw_row))
        
        self.gufw.print_rules(self.gufw.frontend.get_rules())
        self.win_update.destroy()
    
    def _get_same_rules(self, rule_cmd):
        i = 0
        rules_rows = []
        while True:
            try:
                iter_row = self.gufw.rules_model.get_iter(i,)
                cmd = self.gufw.rules_model.get_value(iter_row, 2)
                real_row = self.gufw.rules_model.get_value(iter_row, 14)
                if cmd == rule_cmd:
                    rules_rows.append(real_row)
            except Exception:
                rules_rows = sorted(rules_rows, key=int, reverse=True)
                return rules_rows
            i += 1



# Gufw 20.04.1 - https://costales.github.io/projects/gufw/
# Copyright (C) 2008-2019 Marcos Alvarez Costales https://costales.github.io
#
# Gufw is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Gufw is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Gufw; if not, see http://www.gnu.org/licenses for more
# information.

import glob, os
from gufw.model.ufw_backend import Backend

import gettext
from gettext import gettext as _
gettext.textdomain('gufw')


class Firewall():
    """Set or get the Firewall properties"""
    def __init__(self):
        self.backend = Backend()
        
        self.status = self.backend.get_status()
        self.incoming = self.backend.get_policy('incoming')
        self.outgoing = self.backend.get_policy('outgoing')
        self.routed   = self.backend.get_policy('routed')
        self.ufw_logging = self.backend.get_ufw_logging()
        
        self.gufw_logging = True # By default
        if self.get_cfg_value('GufwLogging') == 'no':
            self.gufw_logging = False
        
        self.profile = ''
        self._user_changed_language() # Rename profile files
        self.all_profiles = self._read_all_profiles()
        self.profile = self._read_default_profile()
        
    # PROFILES
    def get_profile(self):
        return self.profile
    
    def set_profile(self, profile):
        operation = []
        # Force show current rules
        if not self.status:
            self.backend.set_status(True)
        # Remove Gufw rules previous profile
        old_rules = self.get_rules(False)
        ind = len(old_rules)
        for old_row in reversed(old_rules):
            if old_row['command']: # It's a gufw rule
                result = self.backend.delete_rule(str(ind))
                line = _("Deleting previous rules: ") + str(ind) + ' ' + result[0] + ' > ' + result[1].replace('\n', ' | ')
                operation.append(line)
            ind -= 1
        
        # Set new profile in cfg file
        self.profile = profile
        self.set_cfg_value('Profile', profile)
        
        # Get new profile
        (new_status, new_incoming, new_outgoing, new_routed, new_rules) = self.backend.get_profile_values(profile)
        
        # Adding Gufw rules in new profile
        for new_row in new_rules:
            if new_row['command']: # It's a gufw rule
                result = self.backend.add_rule('', # Insert in order
                                                new_row['policy'],
                                                new_row['direction'],
                                                new_row['iface'],
                                                new_row['routed'],
                                                new_row['logging'],
                                                new_row['protocol'],
                                                new_row['from_ip'],
                                                new_row['from_port'],
                                                new_row['to_ip'],
                                                new_row['to_port'])
                line = _("Appending new rules: ") + result[0] + ' > ' + result[1].replace('\n', ' | ')
                operation.append(line)
        
        # New status, incoming, outgoing, routed
        self.status   = new_status
        self.incoming = new_incoming
        self.outgoing = new_outgoing
        if new_routed == 'disabled' and self.backend.get_policy('routed') != 'disabled':
            self.routed = self.backend.get_policy('routed')
        else:
            self.routed = new_routed
        
        return operation
    
    def get_all_profiles(self):
        return self.all_profiles
    
    def add_profile(self, profile):
        self.all_profiles.append(profile)
        self.backend.set_profile_values(profile, self.status, self.incoming, self.outgoing, self.routed, []) # Will create profile
    
    def delete_profile(self, profile):
        self.all_profiles.remove(profile)
        self.backend.delete_file_profile(profile)
    
    def rename_profile(self, old, new):
        if old == self.profile:
            return
        self.backend.rename_file_profile(old, new)
        self.all_profiles[self.all_profiles.index(old)] = new
    
    def import_profile(self, profile_file):
        new_profile = os.path.basename(profile_file)
        new_profile = os.path.splitext(new_profile)[0]
        (status, incoming, outgoing, routed, rules) = self.backend.get_profile_values(profile_file)
        self.add_profile(new_profile)
        self.backend.set_profile_values(new_profile, status, incoming, outgoing, routed, rules)
        
    def export_profile(self, profile_file):
        self.backend.export_profile(self.profile, profile_file)
        
    
    # BASIC
    def get_status(self):
        return self.status
    
    def set_status(self, status):
        self.status = status
        self.backend.set_status(status)
        self.backend.set_profile_values(self.profile, status, self.incoming, self.outgoing, self.routed, self.get_rules(False))
    
    def get_policy(self, policy):
        if policy == 'incoming':
            return self.incoming
        elif policy == 'outgoing':
            return self.outgoing
        elif policy == 'routed':
            if self.backend.get_policy('routed') != 'disabled':
                return self.routed
            else:
                return self.backend.get_policy('routed')
    
    def set_policy(self, policy, value):
        if policy == 'incoming':
            self.incoming = value
            self.backend.set_policy(policy, value)
        elif policy == 'outgoing':
            self.outgoing = value
            self.backend.set_policy(policy, value)
        elif policy == 'routed':
            self.routed = value
            self.backend.set_policy(policy, value)
        
        self.backend.set_profile_values(self.profile, self.status, self.incoming, self.outgoing, self.routed, self.get_rules(True))
    
    def reset(self):
        self.backend.set_profile_values(self.profile, self.status, self.incoming, self.outgoing, self.routed, [])
        self.backend.reset_fw()
    
    # RULES
    def get_rules(self, force_fw_on):
        if self.backend.get_status():
            ufw_rules = self.backend.get_rules()
        else:
            if force_fw_on:
                self.backend.set_status(True)
            ufw_rules = self.backend.get_rules()
            if force_fw_on:
                self.backend.set_status(False)
        profile_rules = self._get_rules_profile()
        return self._compose_rules(ufw_rules, profile_rules)
    
    def get_number_rules(self):
        return self.backend.get_number_rules()
    
    def add_rule(self, description, insert, policy, direction, iface, routed, logging, proto, from_ip, from_port, to_ip, to_port):
        rules_before = self.get_rules(True)
        rules_profile_before = self._get_rules_profile()
        
        cmd = self.backend.add_rule(insert, policy, direction, iface, routed, logging, proto, from_ip, from_port, to_ip, to_port)
        rules_after = self.get_rules(True)
        
        if len(rules_before) != len(rules_after):
            cmd.insert(0, True)
            self._regenerate_file_profile(rules_before, rules_profile_before, rules_after, description, cmd[1], policy, direction, iface, routed, logging, proto, from_ip, from_port, to_ip, to_port)
        else:
            cmd.insert(0, False)
        return cmd # For logging
    
    def delete_rule(self, num):
        rules_before = self.get_rules(True)
        rules_profile_before = self._get_rules_profile()
        result = self.backend.delete_rule(str(num))
        rules_after = self.get_rules(True)
        self._regenerate_file_profile(rules_before, rules_profile_before, rules_after)
        return result # For logging
    
    # LOGGING
    # ufw
    def get_ufw_logging(self):
        return self.ufw_logging

    def set_ufw_logging(self, level):
        self.ufw_logging = level
        self.backend.set_ufw_logging(level)
    
    # Gufw
    def get_logging(self):
        return self.gufw_logging
    
    def set_logging(self, status):
        self.gufw_logging = status
        if status:
            self.set_cfg_value('GufwLogging', 'yes')
        else:
            self.set_cfg_value('GufwLogging', 'no')
    
    def get_log(self):
        return self.backend.get_log()
        
    def add_to_log(self, msg):
        if self.gufw_logging:
            return self.backend.add_to_log(msg)
        return ''
    
    def refresh_log(self):
        self.backend.refresh_log()
    
    
    # LISTENING
    def get_listening_report(self):
        return self.backend.get_listening_report()
    
    
    # CFG FILES
    def get_cfg_value(self, attrib):
        return self.backend.get_cfg_value(attrib)
    
    def set_cfg_value(self, attrib, value):
        self.backend.set_cfg_value(attrib, value)
    
    
    # NET
    def get_internal_ip(self):
        return self.backend.get_net_ip()
    
    def get_net_interfaces(self, exclude_iface=''):
        all_faces = self.backend.get_net_interfaces()
        
        if exclude_iface:
            try:
                all_faces.remove(exclude_iface)
            except Exception:
                pass
        
        return all_faces
    
    # STUFF
    def _read_default_profile(self):
        default_profile = self.get_cfg_value('Profile')
        # Just be sure
        if not default_profile:
            default_profile = _("Home")
        # Error in config file? > Regenerate
        if default_profile and (not default_profile in self.all_profiles):
            self.backend.set_profile_values(default_profile, self.status, self.incoming, self.outgoing, self.routed, [])
            self.all_profiles.append(default_profile)

        return default_profile
    
    def _read_all_profiles(self):
        profiles = []
        while not profiles:
            files = glob.glob('/etc/gufw/*.profile')
            files.sort(key=lambda x: os.path.getctime(x)) # Sort by time and date
            
            for profile in files:
                profiles.append(os.path.splitext(os.path.basename(profile))[0].replace(' ', '_')) # Filename without path and extension
            
            if not profiles: # First run
                self.backend.set_profile_values(_("Public").replace(' ', '_'), True, 'reject',  'allow', 'allow', [])
                self.backend.set_profile_values(_("Office").replace(' ', '_'), True, 'deny',    'allow', 'allow', [])
                self.backend.set_profile_values(_("Home").replace(' ', '_'), self.status, self.incoming, self.outgoing, self.routed, [])
                self.set_cfg_value('Profile', _("Home").replace(' ', '_'))
        
        return profiles
    
    def _user_changed_language(self):
        # Rename profiles file
        profiles = []
        default_profile = self.get_cfg_value('Profile')
        files = glob.glob('/etc/gufw/*.profile')
        files.sort(key=lambda x: os.path.getctime(x)) # Sort by time and date
        
        for profile in files:
            profiles.append(_(os.path.splitext(os.path.basename(profile))[0]).replace(' ', '_')) # Filename without path and extension
        
        for profile in profiles:
            if profile == _("Home") and profile != 'Home':
                self.backend.rename_file_profile('Home', _("Home"))
                self.add_to_log(_("Renamed profile: ") + 'Home' + " > " + _("Home"))
                if default_profile == 'Home':
                    self.set_cfg_value('Profile', _("Home"))
            if profile == _("Office") and profile != 'Office':
                self.backend.rename_file_profile('Office', _("Office"))
                self.add_to_log(_("Renamed profile: ") + 'Office' + " > " + _("Office"))
                if default_profile == 'Office':
                    self.set_cfg_value('Profile', _("Office"))
            if profile == _("Public") and profile != 'Public':
                self.backend.rename_file_profile('Public', _("Public"))
                self.add_to_log(_("Renamed profile: ") + 'Public' + " > " + _("Public"))
                if default_profile == 'Public':
                    self.set_cfg_value('Profile', _("Public"))
    
    def _compose_rules(self, ufw_rules, profile_rules):
        rules = []
        for ufw_rule in ufw_rules:
            rule = {'ufw_rule'   : ufw_rule, # ufw rule
                    'description': '', # description
                    'command'    : '', # command
                    'policy'     : '', # policy
                    'direction'  : '', # direction
                    'protocol'   : '', # proto
                    'from_ip'    : '', # from_ip
                    'from_port'  : '', # from_port
                    'to_ip'      : '', # to_ip
                    'to_port'    : '', # to_port
                    'iface'      : '', # iface
                    'routed'     : '', # routed
                    'logging'    : ''} # logging
            
            for profile_rule in profile_rules:
                if ufw_rule == profile_rule['ufw_rule']:
                    rule = {'ufw_rule'    : profile_rule['ufw_rule'],    # ufw rule
                            'description' : profile_rule['description'], # description
                            'command'     : profile_rule['command'],     # command
                            'policy'      : profile_rule['policy'],      # policy
                            'direction'   : profile_rule['direction'],   # direction
                            'protocol'    : profile_rule['protocol'],    # proto
                            'from_ip'     : profile_rule['from_ip'],     # from_ip
                            'from_port'   : profile_rule['from_port'],   # from_port
                            'to_ip'       : profile_rule['to_ip'],       # to_ip
                            'to_port'     : profile_rule['to_port'],     # to_port
                            'iface'       : profile_rule['iface'],       # iface
                            'routed'      : profile_rule['routed'],      # routed
                            'logging'     : profile_rule['logging']}     # logging
                    break
            
            rules.append(rule)
            
        return rules
    
    def _get_rules_profile(self):
        rules_profile = []
        (status, incoming, outgoing, routed, rules) = self.backend.get_profile_values(self.profile)
        for rule in rules:
            conver_rules = {'ufw_rule'    : rule['ufw_rule'],
                            'description' : rule['description'],
                            'command'     : rule['command'],
                            'policy'      : rule['policy'],
                            'direction'   : rule['direction'],
                            'protocol'    : rule['protocol'],
                            'from_ip'     : rule['from_ip'],
                            'from_port'   : rule['from_port'],
                            'to_ip'       : rule['to_ip'],
                            'to_port'     : rule['to_port'],
                            'iface'       : rule['iface'],
                            'routed'      : rule['routed'],
                            'logging'     : rule['logging']}
            rules_profile.append(conver_rules)
        
        return rules_profile
        
    def _regenerate_file_profile(self, ufw_before, profile_before, ufw_after, description='', cmd='', policy='', direction='', iface='', routed='', logging='', proto='', from_ip='', from_port='', to_ip='', to_port=''):
        """Here there are dragons!"""
        final_rules = []            
        # Operation ufw_before  ufw_after   profile_before  profile_after
        #     +         A         A+B             A               A+B   <-- A completed from profile_before | B completed from parameters
        #     -        A+B         A             A+B               A    
        #     x        A+B        A+C            A+B              A+C   <-- A completed from profile_before | C completed from parameters
        # We have here profile_before + ufw_after + parameters > Just complete!
        for ufw_rule in ufw_after:
            found = False
            for profile_rule in profile_before:
                if ufw_rule['ufw_rule'] == profile_rule['ufw_rule']: # Complete from previous profile
                    found = True
                    new_rule = {'ufw_rule'    : profile_rule['ufw_rule'],
                                'description' : profile_rule['description'],
                                'command'     : profile_rule['command'],
                                'policy'      : profile_rule['policy'],
                                'direction'   : profile_rule['direction'],
                                'protocol'    : profile_rule['protocol'],
                                'from_ip'     : profile_rule['from_ip'],
                                'from_port'   : profile_rule['from_port'],
                                'to_ip'       : profile_rule['to_ip'],
                                'to_port'     : profile_rule['to_port'],
                                'iface'       : profile_rule['iface'],
                                'routed'      : profile_rule['routed'],
                                'logging'     : profile_rule['logging']}
            if not found:
                if ufw_before.count(ufw_rule) == 0: # New: Complete from parameters
                    new_rule = {'ufw_rule'    : ufw_rule['ufw_rule'],
                                'description' : description,
                                'command'     : cmd,
                                'policy'      : policy,
                                'direction'   : direction,
                                'protocol'    : proto,
                                'from_ip'     : from_ip,
                                'from_port'   : from_port,
                                'to_ip'       : to_ip,
                                'to_port'     : to_port,
                                'iface'       : iface,
                                'routed'      : routed,
                                'logging'     : logging}
                else: # Old ufw rule: Nothing
                    continue
                    
            final_rules.append(new_rule) # Just adding rules, a regenerate will be update the rules
        
        self.backend.set_profile_values(self.profile, self.status, self.incoming, self.outgoing, self.routed, final_rules)



# Gufw 20.04.1 - https://costales.github.io/projects/gufw/
# Copyright (C) 2008-2019 Marcos Alvarez Costales https://costales.github.io
#
# Gufw is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Gufw is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Gufw; if not, see http://www.gnu.org/licenses for more
# information.

import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')
gi.require_version('WebKit2', '4.0')
from gi.repository import Gtk, Gdk, WebKit2
from string import Template

import os, re, sys, subprocess, platform

import gettext
from gettext import gettext as _
gettext.textdomain('gufw')

from gufw.view.preferences import Preferences
from gufw.view.add         import Add
from gufw.view.update      import Update
from gufw.view.listening   import ListeningReport
from gufw.view.about       import About


class Gufw:
    GRAY   = '#BAB5AB'
    GREEN  = '#267726'
    RED    = '#DF421E'
    ORANGE = '#D1940C'
    BLUE   = '#314E6C'
    BLACK  = '#000000'

    POLICY2NUM = {'allow': 0,
                  'deny': 1,
                  'reject': 2,
                  'limit': 3,
                  'disabled': 4 }
    POLICY2COLOR = {'allow': RED,
                    'deny': GREEN,
                    'reject': BLUE,
                    'limit': ORANGE,
                    'disabled': GRAY,
                    'others': BLACK}
                     
    NUM2POLICY = {0: 'allow',
                  1: 'deny',
                  2: 'reject',
                  3: 'limit' }
    NUM2DIRECTION = {0: 'in',
                     1: 'out',
                     2: 'both'}
    NUM2PROTO = {0: '',
                 1: 'tcp',
                 2: 'udp' }
    NUM2LOGGING = {0: '',
                   1: 'log',
                   2: 'log-all'}

    def __init__(self, frontend):
        self.frontend = frontend
        
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain('gufw')
        self.builder.add_from_file('/usr/share/gufw/ui/gufw.ui')
        
        self._set_objects_name()
        self._set_initial_values()
        self.winadd = Add(self)
        
        self.builder.connect_signals(self)
        
        Gtk.main()
    
    def _set_objects_name(self):
        self.winMain = self.builder.get_object('main')
        
        self.profile      = self.builder.get_object('profile')
        self.switchStatus = self.builder.get_object('switchStatus')
        self.incoming     = self.builder.get_object('incoming')
        self.outgoing     = self.builder.get_object('outgoing')
        self.routed_lbl   = self.builder.get_object('routed_lbl')
        self.box_routed   = self.builder.get_object('box_routed')
        self.routed       = self.builder.get_object('routed')
        self.shield       = self.builder.get_object('shield')
        
        self.rules_box       = self.builder.get_object('boxRules')
        self.rules           = self.builder.get_object('Rules')
        self.show_add_btn    = self.builder.get_object('btnAddRule')
        self.detele_rule_btn = self.builder.get_object('btnDeleteRule')
        self.edit_rule_btn   = self.builder.get_object('btnEditRule')
        
        self.report_box      = self.builder.get_object('boxReport')
        self.report          = self.builder.get_object('Report')
        self.report_rule_btn = self.builder.get_object('btnReportRule')
        self.report_waiting  = self.builder.get_object('report_waiting')
        
        self.log_box      = self.builder.get_object('boxLog')
        self.log          = self.builder.get_object('log')
        self.log_txt      = self.log.get_buffer()
        self.log_copy     = self.builder.get_object('btnLogCopy')
        self.log_delete   = self.builder.get_object('btnLogCopy')
        
        self.web = self.builder.get_object('boxWeb')
        self.web_content = WebKit2.WebView()
        self.web_content.connect('context-menu', self.context_menu_cb)

        settings = self.web_content.get_settings()
        self.web_content.set_settings(settings)
        self.web.add(self.web_content)
        # For ORCA
        self.tuto_label = Gtk.Label()
        self.tuto_label.set_text(_("Getting started"))
        self.tuto_label.set_mnemonic_widget(self.web)
        
        settings = self.web_content.get_settings()
        settings.set_property('enable-caret-browsing', True)
        
        self.statusbar = self.builder.get_object('statusmsg')
        self.progress  = self.builder.get_object('progress')
        
        # Rules
        self.rules_model = Gtk.ListStore(str,  # 0  ufw rule
                                          str,  # 1  description
                                          str,  # 2  command
                                          str,  # 3  policy
                                          str,  # 4  direction
                                          str,  # 5  proto
                                          str,  # 6  from_ip
                                          str,  # 7  from_port
                                          str,  # 8  to_ip
                                          str,  # 9  to_port
                                          str,  # 10 iface
                                          str,  # 11 routed
                                          str,  # 12 logging
                                          str,  # 13 color
                                          int)  # 14 number (for deleting and updating)
        self.tv_rules = self.rules
        self.tv_rules.set_model(self.rules_model)
        self.tv_rules.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
        
        # Listening report
        self.listening_model = Gtk.ListStore(str, # 0  protocol
                                             int, # 1 port
                                             str, # 2 address
                                             str, # 3 app
                                             str, # 4 color
                                             int) # 5 number
        self.tv_report = self.report
        self.tv_report.set_model(self.listening_model)
        self.tv_report.get_selection().set_mode(Gtk.SelectionMode.SINGLE)
        
        self.txt_tag_green  = self.log_txt.create_tag('color_green',    foreground=self.GREEN)
        self.txt_tag_red    = self.log_txt.create_tag('color_red',      foreground=self.RED)
        self.txt_tag_orange = self.log_txt.create_tag('colored_orange', foreground=self.ORANGE)
        self.txt_tag_blue   = self.log_txt.create_tag('colored_blue',   foreground=self.BLUE)
        self.txt_tag_gray   = self.log_txt.create_tag('colored_gray',   foreground=self.GRAY)
        self.txt_tag_black  = self.log_txt.create_tag('colored_black',  foreground=self.BLACK)
        
        self.btn_report_pause = self.builder.get_object('btnReportPause')
        self.btn_report_play  = self.builder.get_object('btnReportPlay')
        
        # Stack init
        stack = Gtk.Stack()
        stack.set_transition_type(Gtk.StackTransitionType.SLIDE_LEFT_RIGHT)
        stack.set_transition_duration(700)
        # Stack 1 = Tutorial
        vbox_home = Gtk.Box()
        self.viewport_home  = self.builder.get_object('viewport_home')
        self.viewport_home.reparent(vbox_home)
        stack.add_titled(vbox_home, "tutorial", _("Getting started"))
        stack.child_set_property(vbox_home, "icon-name", "go-home") # Set icon
        # Stack 2 = Rules
        vbox_rules = Gtk.Box()
        self.viewport_rules  = self.builder.get_object('viewport_rules')
        self.viewport_rules.reparent(vbox_rules)
        stack.add_titled(vbox_rules, "rules", _("Rules"))
        # Stack 3 = Report
        vbox_report = Gtk.Box()
        self.viewport_report  = self.builder.get_object('viewport_report')
        self.viewport_report.reparent(vbox_report)
        stack.add_titled(vbox_report, "report", _("Report"))
        # Stack 4 = Log
        vbox_log = Gtk.Box()
        self.viewport_log  = self.builder.get_object('viewport_log')
        self.viewport_log.reparent(vbox_log)
        # Translators: Noun
        stack.add_titled(vbox_log, "log", _("Log"))
        # Stack Compose
        stack_switcher = Gtk.StackSwitcher()
        stack_switcher.set_stack(stack)
        # Horizontal center in 1st row
        vbox_1row = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        aspect1 = Gtk.Alignment()
        aspect2 = Gtk.Alignment()
        vbox_1row.pack_start(aspect1, True, True, 0)
        vbox_1row.pack_start(stack_switcher, False, False, 0)
        vbox_1row.pack_start(aspect2, True, True, 0)
        # Pack
        self.stack_vbox = self.builder.get_object('stack_vbox')
        self.stack_vbox.pack_start(vbox_1row, False, True, 0)
        self.stack_vbox.pack_end(stack, True, True, 0)
    
    def _set_initial_values(self):
        for profile in self.frontend.get_all_profiles():
            self.profile.append_text(profile)
        self.profile.set_active(self.frontend.get_all_profiles().index(self.frontend.get_profile()))
        
        self.switchStatus.set_active(self.frontend.get_status())
        
        self.incoming.set_active(self.POLICY2NUM[self.frontend.get_policy('incoming')])
        self.outgoing.set_active(self.POLICY2NUM[self.frontend.get_policy('outgoing')])
        self.routed.set_active(self.POLICY2NUM[self.frontend.get_policy('routed')])
        self.incoming.set_sensitive(self.frontend.get_status())
        self.outgoing.set_sensitive(self.frontend.get_status())
        self.routed.set_sensitive(self.frontend.get_status())
        self.show_add_btn.set_sensitive(self.frontend.get_status())
        self.detele_rule_btn.set_sensitive(self.frontend.get_status())
        self.edit_rule_btn.set_sensitive(self.frontend.get_status())
        self.report_rule_btn.set_sensitive(self.frontend.get_status())
        
        render_txt = Gtk.CellRendererText()
        
        # Rules
        # Translators: Number of  rule
        tree_header = Gtk.TreeViewColumn(_("Nº"), render_txt, text=14, foreground=13)
        tree_header.set_resizable(True)
        tree_header.set_sort_column_id(14)
        tree_header.set_sizing(1)
        self.tv_rules.append_column(tree_header)
        # Translators: Of rule
        tree_header = Gtk.TreeViewColumn(_("Rule"), render_txt, text=0, foreground=13)
        tree_header.set_resizable(True)
        tree_header.set_sort_column_id(0)
        tree_header.set_sizing(1)
        self.tv_rules.append_column(tree_header)
        # Translators: Of rule
        tree_header = Gtk.TreeViewColumn(_("Name"), render_txt, text=1, foreground=13)
        tree_header.set_resizable(True)
        tree_header.set_sort_column_id(1)
        tree_header.set_sizing(1)
        self.tv_rules.append_column(tree_header)
        
        # Listening Report
        # Translators: Number of  rule
        tree_header = Gtk.TreeViewColumn(_("Nº"), render_txt, text=5, foreground=4)
        tree_header.set_resizable(True)
        tree_header.set_sort_column_id(5)
        tree_header.set_sizing(1)
        self.tv_report.append_column (tree_header)
        tree_header = Gtk.TreeViewColumn(_("Protocol"), render_txt, text=0, foreground=4)
        tree_header.set_resizable(True)
        tree_header.set_sort_column_id(0)
        tree_header.set_sizing(1)
        self.tv_report.append_column (tree_header)
        tree_header = Gtk.TreeViewColumn(_("Port"), render_txt, text=1, foreground=4)
        tree_header.set_resizable(True)
        tree_header.set_sort_column_id(1)
        tree_header.set_sizing(1)
        self.tv_report.append_column(tree_header)
        tree_header = Gtk.TreeViewColumn(_("Address"), render_txt, text=2, foreground=4)
        tree_header.set_resizable(True)
        tree_header.set_sort_column_id(2)
        tree_header.set_sizing(1)
        self.tv_report.append_column(tree_header)
        tree_header = Gtk.TreeViewColumn (_("Application"), render_txt, text=3, foreground=4)
        tree_header.set_sort_column_id(3)
        tree_header.set_sizing(1)
        self.tv_report.append_column(tree_header)
        
        self.listening = ListeningReport(self)
        
        self.add_to_log(self.frontend.get_log(), self.GRAY, False)
        
        self._load_tutorial()
        
        self._set_shield()
        
        self._restore_window_size(self.winMain)
        self.winMain.show_all()
        
        if self.frontend.get_policy('routed') == 'disabled':
            self.routed_lbl.set_visible(False)
            self.box_routed.set_visible(False)
        
        self.print_rules(self.frontend.get_rules(False))
        
        self.btn_report_play.hide()

    # Disable the context menu
    def context_menu_cb(webview, context_menu, event, hit_test_result, error):
        return True

    def _load_tutorial(self):
        f = open('/usr/share/gufw/media/tutorial/index.html', 'r')
        html_content = f.read()
        f.close()
        
        replace_html = dict(heading1=_("Getting started"),
                            intro=_("An uncomplicated way to manage your firewall, powered by ufw. Easy, simple, nice and useful! :)"),
                            heading2=_("Basic"),
                            heading3=_("FAQ"),
                            best_conf=_("If you are a normal user, you will be safe with this setting (Status=On, Incoming=Deny, Outgoing=Allow). Remember to append allow rules for your P2P apps:"),
                            rename_profile=_("You can rename your profiles with just 2 clicks on them:"),
                            rule_name=_("The Rule Name will help you to identify your rules in the future:"),
                            faq1_q=_("How to autostart Gufw with the system?"),
                            faq1_a=_("You do not need it. After you do all of the changes in Gufw, the settings are still in place until the next changes."),
                            faq2_q=_("Why is Gufw disabled by default?"),
                            faq2_a=_("By default, the firewall does not open ports to the outside world."),
                            faq3_q=_("Some rules are added by themselves?"),
                            faq3_a=_("Well, the behaviour is such that when you change or import a profile, or when you edit a rule, Gufw will add that rule again, then ufw re-adds that rule for IPv4 and IPv6."),
                            faq4_q=_("What is Allow, Deny, Reject and Limit?"),
                            faq4_a1=_("Allow: Will allow traffic."),
                            faq4_a2=_("Deny: Will deny traffic."),
                            faq4_a3=_("Reject: Will deny traffic and will inform that it has been rejected."),
                            faq4_a4=_("Limit: Will deny traffic if an IP tried several connections."),
                            faq5_q=_("I see some rules in all profiles"),
                            faq5_a=_("All the ufw rules will be appear in all profiles."),
                            faq6_q=_("What do I see in the Listening Report?"),
                            faq6_a=_("The ports on the live system in the listening state for TCP and the open state for UDP."),
                            faq8_q=_("I want even more!"),
                            faq8_a=_("You'll find more information in the community documentation :)"))
        html = Template(html_content).safe_substitute(replace_html)
        self.web_content.load_html(html, "file:///")
    
    def _show_web(self, url):
        distro = platform.linux_distribution()[0].lower()
        try:
            user = sys.argv[1]
        except Exception:
            self.show_dialog(self.winMain, _("Visit this web (please, copy & paste in your browser):"), url)
            return
        
        if distro != 'ubuntu' and distro != 'linuxmint' and distro != 'debian':
            self.show_dialog(self.winMain, _("Visit this web (please, copy & paste in your browser):"), url)
            return
        if user == 'root' or user == '-ssh' or not user:
            self.show_dialog(self.winMain, _("Visit this web (please, copy & paste in your browser):"), url)
            return
        
        # Launching browser
        cmd = "su -c 'python -m webbrowser -t \"" + url + "\"' - " + user
        subprocess.Popen(cmd, shell=True)
    
    def on_menu_import_activate(self, widget, data=None):
        import_profile = self._file_dialog('open', _("Import Profile"))
        
        profile = os.path.basename(import_profile) #Filename
        profile = os.path.splitext(profile)[0] # Ext
        
        if not profile:
            self.set_statusbar_msg(_("Import cancelled"))
            return
        
        if oct(os.stat(import_profile).st_mode & 0o777) != oct(0o600):
            self.show_dialog(self.winMain, _("Error"), _("Filename has wrong permissions (not 600). Trust only on your exported profiles"))
            return
        
        if not re.match('^[A-Za-z0-9_-]*$', profile):
            self.show_dialog(self.winMain, _("Error"), _("Filename has not valid characters. Rename the file\nto only letters, numbers, dashes and underscores"))
            return
        
        if profile in self.frontend.get_all_profiles():
            self.set_statusbar_msg(_("Operation cancelled"))
            self.show_dialog(self.winMain, _("Profile already exists"), _("Delete it before from the Preferences Window or rename the file (the profile will be the filename)"))
        else:
            self.frontend.import_profile(import_profile)
            self.profile.append_text(profile)
            self.add_to_log(_("Profile imported: ") + import_profile)
            self.set_statusbar_msg(_("Profile imported, now you can choose it in the profiles"))
    
    def on_menu_export_activate(self, widget, data=None):
        export_profile = self._file_dialog('save', _("Export Profile"))
        
        if not export_profile:
            self.set_statusbar_msg(_("Export cancelled"))
            return
        
        if export_profile[-8:] != '.profile':
            export_profile = export_profile + '.profile'
        
        self.frontend.export_profile(export_profile)
        self.add_to_log(_("Profile exported: ") + export_profile)
        self.set_statusbar_msg(_("Profile exported"))
    
    def on_main_delete_event(self, widget, data=None):
        self._exit_gufw()
    
    def on_menu_quit_activate(self, widget, data=None):
        self._exit_gufw()
    
    def _exit_gufw(self):
        self._save_window_size(self.winMain)
        self.listening.stopping()
        Gtk.main_quit()
    
    def _set_shield(self):
        if self.frontend.get_status():
            file_shield = os.path.join('/usr/share/gufw/media/shields/', self.frontend.get_policy('incoming').lower() + '_' + self.frontend.get_policy('outgoing').lower() + '_' + self.frontend.get_policy('routed').lower() + '.png')
        else:
            if self.frontend.get_policy('routed') == 'disabled':
                file_shield = os.path.join('/usr/share/gufw/media/shields/', 'disabled_disabled_disabled.png')
            else:
                file_shield = os.path.join('/usr/share/gufw/media/shields/', 'disabled_disabled_enabled.png')
        self.shield.set_from_file(file_shield)
    
    def on_menu_reset_activate(self, widget, data=None):
        answer = self._show_question(self.winMain, _("Reset Firewall"), _("This will remove all rules in the current\nprofile and disable the firewall"), _("Do you want to continue?"))
        if answer:
            if self.frontend.get_status():
                self.switchStatus.set_active(False)
            self.frontend.reset()
            self.add_to_log(_("Removed rules and reset firewall!"))
    
    def on_btnLogRemove_clicked(self, widget, data=None):
        self.frontend.refresh_log()
        self.log_txt.set_text('')
        self.add_to_log(_("Gufw Log: Removed"))
        self.set_statusbar_msg(_("Gufw Log removed"))
    
    def on_btnLogCopy_clicked(self, widget, data=None):
        self.clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
        self.clipboard.set_text(self.frontend.get_log(), -1)
        self.set_statusbar_msg(_("Text copied to clipboard"))
    
    def on_menu_about_activate(self, widget, data=None):
        aboutwin = About(self)
    
    def on_menu_preferences_activate(self, widget, data=None):
        preferenceswin = Preferences(self)
    
    def on_incoming_changed(self, widget, data=None):
        self.frontend.set_policy('incoming', self.NUM2POLICY[self.incoming.get_active()].lower())
        self._set_shield()
        # Translators: Policy
        self.add_to_log(_("Incoming: ") + self.incoming.get_active_text(), self.POLICY2COLOR[self.frontend.get_policy('incoming')])
        self.set_statusbar_msg(_("Incoming policy changed"))
        if self.frontend.get_policy('incoming') != self.NUM2POLICY[self.incoming.get_active()].lower():
            self.show_dialog(self.winMain, _("There was an error changing the incoming policy"), _("Restart your firewall to refresh to the real status\nand please report this bug"))
    
    def on_outgoing_changed(self, widget, data=None):
        self.frontend.set_policy('outgoing', self.NUM2POLICY[self.outgoing.get_active()].lower())
        self._set_shield()
        self.add_to_log(_("Outgoing: ") + self.outgoing.get_active_text(), self.POLICY2COLOR[self.frontend.get_policy('outgoing')])
        self.set_statusbar_msg(_("Outgoing policy changed"))
        if self.frontend.get_policy('outgoing') != self.NUM2POLICY[self.outgoing.get_active()].lower():
            self.show_dialog(self.winMain, _("There was an error changing the outgoing policy"), _("Restart your firewall to refresh to the real status\nand please report this bug"))
    
    def on_routed_changed(self, widget, data=None):
        try: # Could be inconsistent between imports and changes of Routed status
            self.frontend.set_policy('routed', self.NUM2POLICY[self.routed.get_active()].lower())
        except Exception:
            return
        self._set_shield()
        # Translators: Routed firewall changed
        self.add_to_log(_("Routed: ") + self.routed.get_active_text(), self.POLICY2COLOR[self.frontend.get_policy('routed')])
        self.set_statusbar_msg(_("Routed policy changed"))
        if self.frontend.get_policy('routed') != self.NUM2POLICY[self.routed.get_active()].lower():
            self.show_dialog(self.winMain, _("There was an error changing the routed policy"), _("Restart your firewall to refresh to the real status\nand please report this bug"))
    
    def on_btnAddRule_clicked(self, widget, data=None):
        self.show_add_btn.set_sensitive(False)
        self.edit_rule_btn.set_sensitive(False)
        self.report_rule_btn.set_sensitive(False)
        self.winadd.show_win()
    
    def on_btnReportRule_clicked(self, widget, data=None):
        select_this_row = -1
        protocol = port = address = app = ''
        (model, rows) = self.tv_report.get_selection().get_selected_rows()
        if len(rows) == 1:
            iter_row = self.listening_model.get_iter(rows[0],)
            protocol = self.listening_model.get_value(iter_row, 0)
            port     = str(self.listening_model.get_value(iter_row, 1))
            address  = self.listening_model.get_value(iter_row, 2)
            app      = self.listening_model.get_value(iter_row, 3)
            if address == '*':
                address = ''
        else:
            self.show_dialog(self.winMain, _("Select just one row"), _("You can create a rule from just one row selected"))
            return
        
        self.show_add_btn.set_sensitive(False)
        self.edit_rule_btn.set_sensitive(False)
        self.report_rule_btn.set_sensitive(False)
        self.winadd.set_add_from_report(protocol, port, address, app)
        self.winadd.show_win()        
    
    def on_btnReportPause_clicked(self, widget, data=None):
        self.btn_report_pause.hide()
        self.btn_report_play.show()
        self.listening.set_pause(True)
    
    def on_btnReportPlay_clicked(self, widget, data=None):
        self.btn_report_pause.show()
        self.btn_report_play.hide()
        self.listening.set_pause(False)
    
    def on_switchStatus_active_notify(self, widget, data=None):
        self.frontend.set_status(self.switchStatus.get_active())
        self.print_rules(self.frontend.get_rules(False))
        
        if self.frontend.get_status():
            # Translators: About firewall
            self.add_to_log(_("Status: Enabled"), self.GREEN)
            self.set_statusbar_msg(_("Firewall enabled"))
        else:
            # Translators: About firewall
            self.add_to_log(_("Status: Disabled"), self.RED)
            self.set_statusbar_msg(_("Firewall disabled"))
        
        self.incoming.set_sensitive(self.frontend.get_status())
        self.outgoing.set_sensitive(self.frontend.get_status())
        self.routed.set_sensitive(self.frontend.get_status())
        self.show_add_btn.set_sensitive(self.frontend.get_status())
        self.detele_rule_btn.set_sensitive(self.frontend.get_status())
        self.edit_rule_btn.set_sensitive(self.frontend.get_status())
        self.report_rule_btn.set_sensitive(self.frontend.get_status())
        self._set_shield()
        if self.frontend.get_status() != self.switchStatus.get_active():
            self.show_dialog(self.winMain, _("There was an error changing the firewall status"), _("Restart your firewall to refresh to the real status and please report this bug"))

    def on_btnDeleteRule_clicked(self, widget, data=None):
        total_rules = []
        rules_selected = False
        (model, rows) = self.tv_rules.get_selection().get_selected_rows()
        
        # Compose real numbers
        for row in rows:
            iter_row = self.rules_model.get_iter(row,)
            total_rules.append(self.rules_model.get_value(iter_row, 14))
        total_rules = sorted(total_rules, key=int, reverse=True)
        
        # Confirm?
        if (self.frontend.get_config_value('ConfirmDeteleRule') == 'yes') and (len(total_rules) > 0):
            answer = self._show_question(self.winMain, _("Delete rule"), _("You will delete all selected rules"), _("Do you want to continue?"))
            if not answer:
                return
        
        for row in total_rules:
            rules_selected = True            
            rules_before = self.frontend.get_rules()
            cmd = self.frontend.delete_rule(row)
            rules_after = self.frontend.get_rules()
            
            if len(rules_before) != len(rules_after):
                self.add_to_log(cmd[0])
                self.set_statusbar_msg(_("Rule(s) deleted"))
            else:
                # Translators: Error running an ufw command
                self.add_to_log(_("Error running: ") + cmd[0] + ' > ' + cmd[1].replace('\n', ' | '))
                self.set_statusbar_msg(_("Error. Review Gufw Log"))
        
        if rules_selected:
            self.print_rules(self.frontend.get_rules())
        else:
            self.show_dialog(self.winMain, _("No rule selected"), _("You have to select a rule"))
    
    def _get_total_rows(self, model):
        i = 0
        while True:
            try:
                iter_row = model.get_iter(i,)
            except Exception:
                return i
            i += 1
    
    def on_btnEditRule_clicked(self, widget, data=None):
        (model, rows) = self.tv_rules.get_selection().get_selected_rows()
        
        if len(rows) != 1:
            self.show_dialog(self.winMain, _("Select just one row"), _("You can edit just one rule"))
            return
        
        # Just one rule selected
        iter_row = self.rules_model.get_iter(rows[0],)
        cmd = self.rules_model.get_value(iter_row, 2)
        
        if not cmd: # ufw rule > inmutable
            self.show_dialog(self.winMain, _("Immutable Rule"), _("You can't edit a rule added from ufw"))
            return
        
        description = self.rules_model.get_value(iter_row, 1)
        policy      = self.rules_model.get_value(iter_row, 3)
        direction   = self.rules_model.get_value(iter_row, 4)
        proto       = self.rules_model.get_value(iter_row, 5)
        from_ip     = self.rules_model.get_value(iter_row, 6)
        from_port   = self.rules_model.get_value(iter_row, 7)
        to_ip       = self.rules_model.get_value(iter_row, 8)
        to_port     = self.rules_model.get_value(iter_row, 9)
        iface       = self.rules_model.get_value(iter_row, 10)
        routed      = self.rules_model.get_value(iter_row, 11)
        logging     = self.rules_model.get_value(iter_row, 12)
        ufw_row     = self.rules_model.get_value(iter_row, 14)
        
        updatewin = Update(self, ufw_row, description, cmd, policy, direction, proto, from_ip, from_port, to_ip, to_port, iface, routed, logging)

    def on_profile_changed(self, widget, data=None):
        operation = self.frontend.set_profile(self.profile.get_active_text())
        self.add_to_log(_("Changing profile: ") + self.profile.get_active_text())
        
        self.incoming.set_active(self.POLICY2NUM[self.frontend.get_policy('incoming')])
        self.outgoing.set_active(self.POLICY2NUM[self.frontend.get_policy('outgoing')])
        self.routed.set_active(self.POLICY2NUM[self.frontend.get_policy('routed')])
        self.switchStatus.set_active(self.frontend.get_status())
        
        self.print_rules(self.frontend.get_rules())
        
        for msg in operation:
            self.add_to_log(msg)
    
    def show_dialog(self, win, header, msg):
        dialog = Gtk.MessageDialog(win, 0, Gtk.MessageType.INFO, Gtk.ButtonsType.OK, header)
        dialog.format_secondary_text(msg)
        dialog.run()
        dialog.destroy()
        
    def _show_question(self, win, header, msg, question):
        reset_dialog = Gtk.MessageDialog(win,
                       Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
                       Gtk.MessageType.WARNING, Gtk.ButtonsType.NONE,
                       msg)
        reset_dialog.format_secondary_markup(question)
        reset_dialog.set_title(header)
        reset_dialog.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_YES, Gtk.ResponseType.YES)
        reset_answer = reset_dialog.run()
        reset_dialog.destroy()
        if reset_answer == Gtk.ResponseType.YES:
            return True
        else:
            return False
    
    def _get_log_color(self, color):
        if color == self.GREEN:
            return self.txt_tag_green
        if color == self.RED:
            return self.txt_tag_red
        if color == self.ORANGE:
            return self.txt_tag_orange
        if color == self.BLUE:
            return self.txt_tag_blue
        if color == self.GRAY:
            return self.txt_tag_gray
        
        return self.txt_tag_black
            
    def add_to_log(self, msg, color=BLACK, force_save=True):
        if not self.frontend.get_logging():
            return
        
        if force_save:
            new_line = self.frontend.add_to_log(msg)
            self.log_txt.insert_with_tags(self.log_txt.get_start_iter(), new_line, self._get_log_color(color)) 
        else:
            self.log_txt.insert_with_tags(self.log_txt.get_end_iter(), msg, self._get_log_color(color)) 
    
    def set_statusbar_msg(self, msg):
        cid = self.statusbar.get_context_id('default context')
        self.statusbar.push(cid, msg)
    
    def print_rules(self, rules):
        self.rules_model.clear()
        if not self.frontend.get_status():
            return
        
        row = 1
        for rule in (rules):
            iter_row = self.rules_model.insert(row)
            # Translators: ufw string
            translated_rule = rule['ufw_rule'].replace(" ALLOW ",  _(" ALLOW "))
            # Translators: ufw string
            translated_rule = translated_rule.replace(" DENY ",    _(" DENY "))
            # Translators: ufw string
            translated_rule = translated_rule.replace(" REJECT ",  _(" REJECT "))
            # Translators: ufw string
            translated_rule = translated_rule.replace(" LIMIT ",   _(" LIMIT "))
            # Translators: ufw string
            translated_rule = translated_rule.replace(" OUT ",     _(" OUT "))
            # Translators: ufw string
            translated_rule = translated_rule.replace(" IN ",      _(" IN "))
            # Translators: ufw string
            translated_rule = translated_rule.replace(" FWD ",     _(" FWD "))
            # Translators: ufw string
            translated_rule = translated_rule.replace("Anywhere",  _("Anywhere"))
            # Translators: ufw string
            translated_rule = translated_rule.replace(" (log) ",     _("(log)"))
            # Translators: ufw string
            translated_rule = translated_rule.replace(" (log-all) ", _("(log-all)"))
            # Translators: ufw string
            translated_rule = translated_rule.replace(" (out)",    _(" (out)"))
            # Translators: ufw string
            translated_rule = translated_rule.replace(" on ",      _(" on "))
            
            self.rules_model.set_value(iter_row, 0,  translated_rule)     # ufw rule
            self.rules_model.set_value(iter_row, 1,  rule['description']) # description
            self.rules_model.set_value(iter_row, 2,  rule['command'])     # command
            self.rules_model.set_value(iter_row, 3,  rule['policy'])      # policy
            self.rules_model.set_value(iter_row, 4,  rule['direction'])   # direction
            self.rules_model.set_value(iter_row, 5,  rule['protocol'])    # proto
            self.rules_model.set_value(iter_row, 6,  rule['from_ip'])     # from_ip
            self.rules_model.set_value(iter_row, 7,  rule['from_port'])   # from_port
            self.rules_model.set_value(iter_row, 8,  rule['to_ip'])       # to_ip
            self.rules_model.set_value(iter_row, 9,  rule['to_port'])     # to_port
            self.rules_model.set_value(iter_row, 10, rule['iface'])       # iface
            self.rules_model.set_value(iter_row, 11, rule['routed'])      # routed
            self.rules_model.set_value(iter_row, 12, rule['logging'])     # logging
            self.rules_model.set_value(iter_row, 14, row)                 # number
            
            if 'ALLOW' in rule['ufw_rule']:
                self.rules_model.set_value(iter_row, 13, self.POLICY2COLOR['allow'])  # color
            elif 'DENY' in rule['ufw_rule']:
                self.rules_model.set_value(iter_row, 13, self.POLICY2COLOR['deny'])   # color
            elif 'REJECT' in rule['ufw_rule']:
                self.rules_model.set_value(iter_row, 13, self.POLICY2COLOR['reject']) # color
            elif 'LIMIT' in rule['ufw_rule']:
                self.rules_model.set_value(iter_row, 13, self.POLICY2COLOR['limit'])  # color
            else:
                self.rules_model.set_value(iter_row, 13, self.POLICY2COLOR['others']) # color
            
            row += 1
    
    def _file_dialog(self, type_dialog, title):
        if type_dialog == 'open':
            type_win = Gtk.FileChooserAction.OPEN
            stock_win = Gtk.STOCK_OPEN
        else:
            type_win = Gtk.FileChooserAction.SAVE
            stock_win = Gtk.STOCK_SAVE
        
        dialog = Gtk.FileChooserDialog(title, None,
                                       type_win,
                                      (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                                       stock_win, Gtk.ResponseType.OK))
        filter_profile = Gtk.FileFilter()
        filter_profile.set_name(_("Gufw profile"))
        filter_profile.add_pattern("*.profile")
        dialog.add_filter(filter_profile)
        filter_all = Gtk.FileFilter()
        filter_all.set_name(_("All files"))
        filter_all.add_pattern('*')
        dialog.add_filter(filter_all)
        dialog.set_do_overwrite_confirmation(True)
        
        response = dialog.run()
        select = ''
        if response == Gtk.ResponseType.OK:
            select = dialog.get_filename()
        
        dialog.destroy()    
        return select
    
    def _restore_window_size(self, win):
        # Fix GUI for small screens:
        if Gdk.Screen.height() <= 600:
            win.set_size_request(365, 480)
        if Gdk.Screen.height() <= 480:
            win.set_size_request(365, 390)
        
        width = 0
        height = 0
        if self.frontend.get_config_value('WindowWidth'):
            width = int(self.frontend.get_config_value('WindowWidth'))
        if self.frontend.get_config_value('WindowHeight'):
            height = int(self.frontend.get_config_value('WindowHeight'))
        
        if width >= Gdk.Screen.width() and height >= Gdk.Screen.height():
            win.maximize()
        elif width != 0 and height != 0:
            win.resize(width, height)            
            
    def _save_window_size(self, win):
        width, height = win.get_size()
        self.frontend.set_config_value('WindowWidth',  str(width))
        self.frontend.set_config_value('WindowHeight', str(height))
    
    def validate_rule(self, win, from_ip, from_port, to_ip, to_port, insert='', routed=''):
        # At least 1 Port/IP
        if not from_ip and not from_port and not to_ip and not to_port and routed == "Not Forward":
            self.show_dialog(win, _("Insert IP/Ports"), _("You need to insert IP/ports in to/from fields"))
            return False
        
        # Not allow insert number bigger that number of rules
        if not insert:
            insert_into = 0
        else:
            insert_into = int(insert)
        if insert_into > 0 and insert_into > self.frontend.get_number_rules():
            self.show_dialog(win, _("Insert number bigger that number of rules"), _("By example, if you have 3 rules, you can't insert a rule into position 4"))
            return False
        
        return True



# Gufw 20.04.1 - https://costales.github.io/projects/gufw/
# Copyright (C) 2008-2019 Marcos Alvarez Costales https://costales.github.io
#
# Gufw is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Gufw is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Gufw; if not, see http://www.gnu.org/licenses for more
# information.

import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')
from gi.repository import Gtk
from gi.repository.Gdk import Color
import os, glob, configparser

import gettext
from gettext import gettext as _
gettext.textdomain('gufw')

DIR_PROFILES = '/etc/gufw/app_profiles'


class Add:
    def __init__(self, gufw):
        self.gufw = gufw
        
        self.apps = AppProfiles()
        
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain('gufw')
        self.builder.add_from_file('/usr/share/gufw/ui/add.ui')
        
        self._set_objects_name()
        self._set_initial_values()
        
        self.win_add.set_transient_for(gufw.winMain)
        self.updating_subcategories = False
        
        self.builder.connect_signals(self)
    
    def _set_objects_name(self):
        self.win_add = self.builder.get_object('AddRule')
        self.tabs    = self.builder.get_object('addTabs')
        self.add_btn = self.builder.get_object('btnAddRuleWin')
        
        self.preconfig_policy      = self.builder.get_object('preconfig_policy')
        self.preconfig_direction   = self.builder.get_object('preconfig_direction')
        self.preconfig_category    = self.builder.get_object('preconfig_category')
        self.app_filter            = self.builder.get_object('app_filter')
        self.preconfig_subcategory = self.builder.get_object('preconfig_subcategory')
        self.preconfig_app         = self.builder.get_object('preconfig_app')
        self.preconfig_description = self.builder.get_object('preconfig_description')
        self.img_description       = self.builder.get_object('img_description')
        self.warningbox            = self.builder.get_object('warningbox')
        self.lbl_inforule          = self.builder.get_object('lbl_preconfig_info')
        
        self.simple_rule_name = self.builder.get_object('simple_rule_name')
        self.simple_policy    = self.builder.get_object('simple_policy')
        self.simple_direction = self.builder.get_object('simple_direction')
        self.simple_protocol  = self.builder.get_object('simple_protocol')
        self.simple_port      = self.builder.get_object('simple_port')
        
        self.advanced_rule_name  = self.builder.get_object('advanced_rule_name')
        self.advanced_insert     = self.builder.get_object('advanced_insert')
        self.advanced_policy     = self.builder.get_object('advanced_policy')
        self.advanced_direction  = self.builder.get_object('advanced_direction')
        self.advanced_iface      = self.builder.get_object('advanced_iface')
        self.advanced_routed_img = self.builder.get_object('routed_img')
        self.advanced_routed     = self.builder.get_object('advanced_routed')
        self.advanced_log        = self.builder.get_object('advanced_log')
        self.advanced_protocol   = self.builder.get_object('advanced_protocol')
        self.advanced_from_ip    = self.builder.get_object('advanced_from_ip')
        self.advanced_from_port  = self.builder.get_object('advanced_from_port')
        self.advanced_to_ip      = self.builder.get_object('advanced_to_ip')
        self.advanced_to_port    = self.builder.get_object('advanced_to_port')
    
    def _set_initial_values(self):
        self._set_app_combobox()
        
        # Bug gnome #549478
        self.preconfig_policy.set_active(0)
        self.preconfig_direction.set_active(0)
        
        self.simple_policy.set_active(0)
        self.simple_direction.set_active(0)
        self.simple_protocol.set_active(0)
        
        self.advanced_policy.set_active(0)
        self.advanced_direction.set_active(0)
        # Translators: About the network interfaces in the OS
        self.advanced_iface.append_text(_("All Interfaces"))
        # Translators: About traffic
        self.advanced_routed.append_text(_("Not Forward"))
        for ifaceName in self.gufw.frontend.get_net_interfaces():
            self.advanced_iface.append_text(ifaceName)
            self.advanced_routed.append_text(ifaceName)
        self.advanced_iface.set_active(0)
        self.advanced_routed.set_active(0)
        self.advanced_log.set_active(0)
        self.advanced_protocol.set_active(0)
        
        if self.gufw.frontend.get_policy('routed') == 'disabled':
            self.advanced_routed_img.set_visible(False)
            self.advanced_routed.set_visible(False)
    
    def show_win(self):
        if self.app_filter.get_text():
            self.app_filter.set_text('')
        self.win_add.show()
        
    
    def set_add_from_report(self, protocol='', port='', address='', name=''):
        self.advanced_rule_name.set_text(name)
        if protocol[:3] == 'TCP':
            self.advanced_protocol.set_active(1)
        elif protocol[:3] == 'UDP':
            self.advanced_protocol.set_active(2)
        else:
            self.advanced_protocol.set_active(0)
        self.advanced_to_ip.set_text(address)
        self.advanced_to_port.set_text(port)        
        self.tabs.set_current_page(2)
        
    def _hide_win(self):
        self.win_add.hide()
    
    def _set_app_combobox(self):
        categories = self.apps.get_just_categories()
        for cat in categories:
            self.preconfig_category.append_text(cat)
        try:
            # Translators: ALL applications
            self.preconfig_category.set_active(categories.index(_("All")))
        except Exception:
            self.preconfig_category.set_active(0)
        
        subcategories = self.apps.get_subcategories(self.preconfig_category.get_active_text())
        for subcat in subcategories:
            self.preconfig_subcategory.append_text(subcat)
        self.preconfig_subcategory.set_active(0)
        
        apps = self.apps.get_apps_cat_subcat(self.preconfig_category.get_active_text(), self.preconfig_subcategory.get_active_text())
        for app in apps:
            self.preconfig_app.append_text(app)
        self.preconfig_app.set_active(0)
        
        tooltip = self.apps.get_app(self.preconfig_app.get_active_text())[0] + '\n' + _("Ports: ") + self.apps.get_app(self.preconfig_app.get_active_text())[1]
        self.img_description.set_tooltip_text(tooltip)
    
    def on_btnAddClose_clicked(self, widget, data=None):
        self._win_close()
    
    def on_AddRule_delete_event(self, widget, data=None):
        self._win_close()
        return True # Overwrite event
    
    def _win_close(self):
        self.gufw.show_add_btn.set_sensitive(self.gufw.frontend.get_status())
        self.gufw.edit_rule_btn.set_sensitive(self.gufw.frontend.get_status())
        self.gufw.report_rule_btn.set_sensitive(self.gufw.frontend.get_status())
        self._hide_win()
    
    def on_copy_from_IP_clicked(self, widget, data=None):
        self.advanced_from_ip.set_text(self.gufw.frontend.get_internal_ip())
    
    def on_copy_to_IP_clicked(self, widget, data=None):
        self.advanced_to_ip.set_text(self.gufw.frontend.get_internal_ip())
    
    def on_copy_simple_to_advanced_clicked(self, widget, data=None):
        self.advanced_rule_name.set_text(self.simple_rule_name.get_text())
        self.advanced_policy.set_active(self.simple_policy.get_active())
        self.advanced_direction.set_active(self.simple_direction.get_active())
        self.advanced_protocol.set_active(self.simple_protocol.get_active())
        self.advanced_to_port.set_text(self.simple_port.get_text())
        self.tabs.set_current_page(2)
    
    def on_copy_preconfig_to_advanced_clicked(self, widget, data=None):
        self.advanced_rule_name.set_text(self.preconfig_app.get_active_text())
        self.advanced_policy.set_active(self.preconfig_policy.get_active())
        self.advanced_direction.set_active(self.preconfig_direction.get_active())
        self.advanced_protocol.set_active(0)
        self.advanced_to_port.set_text(self.apps.get_app(self.preconfig_app.get_active_text())[1])
        self.tabs.set_current_page(2)
    
    def _set_advanced_addBtn_control(self, ntab):
        # Not allow Both Protocol with a range of ports in Simple and Advanced tab
        if ntab == 2 and (':' in self.advanced_to_port.get_text() or ':' in self.advanced_from_port.get_text()) and not self.advanced_protocol.get_active():
            self.add_btn.set_sensitive(False)
            self.add_btn.set_tooltip_text(_("Choose a TCP or UDP Protocol with a range of ports"))
        else:
            self.add_btn.set_sensitive(True)
            self.add_btn.set_tooltip_text('')
    
    def _set_simple_addBtn_control(self, ntab):
        # Not allow Both Protocol with a range of ports in Simple and Advanced tab
        if ntab == 1 and ':' in self.simple_port.get_text() and not self.simple_protocol.get_active():
            self.add_btn.set_sensitive(False)
            self.add_btn.set_tooltip_text(_("Choose a TCP or UDP Protocol with a range of ports"))
        else:
            self.add_btn.set_sensitive(True)
            self.add_btn.set_tooltip_text('')
    
    def on_addTabs_switch_page(self, widget, data=None, change_to_ntab=None):
        if change_to_ntab == 1:
            self._set_simple_addBtn_control(change_to_ntab)
        elif change_to_ntab == 2:
            self._set_advanced_addBtn_control(change_to_ntab)
        else:
            self.add_btn.set_sensitive(True)
            self.add_btn.set_tooltip_text('')
    
    def on_simple_protocol_changed(self, widget, data=None):
        self._set_simple_addBtn_control(self.tabs.get_current_page())
    
    def on_simple_port_changed(self, widget, data=None):
        self._set_simple_addBtn_control(self.tabs.get_current_page())
    
    def on_advanced_protocol_changed(self, widget, data=None):
        self._set_advanced_addBtn_control(self.tabs.get_current_page())
    
    def on_advanced_from_port_changed(self, widget, data=None):
        if '/' in self.advanced_from_port.get_text():
            self._set_from_port_sensitive(False)
        else:
            self._set_from_port_sensitive(True)
        
        self._set_advanced_addBtn_control(self.tabs.get_current_page())
    
    def on_advanced_to_port_changed(self, widget, data=None):
        if '/' in self.advanced_to_port.get_text():
            self._set_to_port_sensitive(False)
        else:
            self._set_to_port_sensitive(True)
        
        self._set_advanced_addBtn_control(self.tabs.get_current_page())
    
    def _set_from_port_sensitive(self, value=True):
        self.advanced_protocol.set_sensitive(value)
        self.advanced_from_ip.set_sensitive(value)
        self.advanced_to_ip.set_sensitive(value)
        self.advanced_to_port.set_sensitive(value)
    
    def _set_to_port_sensitive(self, value=True):
        self.advanced_protocol.set_sensitive(value)
        self.advanced_from_ip.set_sensitive(value)
        self.advanced_to_ip.set_sensitive(value)
        self.advanced_from_port.set_sensitive(value)
    
    def on_preconfig_category_changed(self, widget, data=None):
        self.updating_subcategories = True
        self.app_filter.set_text('')
        self.preconfig_subcategory.set_sensitive(False)
        self.preconfig_app.set_sensitive(False)
        # Delete all subcategories & apps
        self.preconfig_subcategory.remove_all()
        self.preconfig_app.remove_all()
        # Append new subcategories
        subcategories = self.apps.get_subcategories(self.preconfig_category.get_active_text())
        for subcat in subcategories:
            self.preconfig_subcategory.append_text(subcat)
        # Append new apps
        # Translators: ALL applications
        apps = self.apps.get_apps_cat_subcat(self.preconfig_category.get_active_text(), _("All"))
        for app in apps:
            self.preconfig_app.append_text(app)
        # Set initial subcat and app
        self.preconfig_subcategory.set_active(0)
        self.preconfig_app.set_active(0)

        self.preconfig_subcategory.set_sensitive(True)
        self.preconfig_app.set_sensitive(True)
        self.updating_subcategories = False
    
    def on_preconfig_subcategory_changed(self, widget, data=None):
        if self.updating_subcategories:
            return
        self.updating_subcategories = True
        self.app_filter.set_text('')
        self.preconfig_app.set_sensitive(False)
        # Delete all apps
        self.preconfig_app.remove_all()
        # Append new entries
        apps = self.apps.get_apps_cat_subcat(self.preconfig_category.get_active_text(), self.preconfig_subcategory.get_active_text())
        for app in apps:
            self.preconfig_app.append_text(app)
        # Set initial app
        self.preconfig_app.set_active(0)
        self.preconfig_app.set_sensitive(True)
        self.updating_subcategories = False
    
    def on_app_filter_search_changed(self, widget, data=None):
        if self.updating_subcategories:
            return
        
        user_filter = self.app_filter.get_text().lower()
        if not user_filter:
            # Refresh apps
            self.preconfig_app.remove_all()
            apps = self.apps.get_apps_cat_subcat(self.preconfig_category.get_active_text(), self.preconfig_subcategory.get_active_text())
            for app in apps:
                self.preconfig_app.append_text(app)
            
            self.preconfig_app.set_active(0)
            self.app_filter.modify_fg(Gtk.StateFlags.NORMAL, None)
            return
        
        # Apps
        new_apps = []
        apps = self.apps.get_apps_cat_subcat(self.preconfig_category.get_active_text(), self.preconfig_subcategory.get_active_text())
        for app in apps:
            if (app.lower().find(user_filter) != -1 or                       # Search in name
                self.apps.get_app(app)[0].lower().find(user_filter) != -1 or # Search in description
                self.apps.get_app(app)[1].lower().find(user_filter) != -1):  # Search in ports
                if new_apps.count(app) == 0:
                    new_apps.append(app)
        
        if len(new_apps) > 0:
            self.preconfig_app.remove_all()
            for app in new_apps:
                self.preconfig_app.append_text(app)
            self.preconfig_app.set_active(0)
            self.app_filter.modify_fg(Gtk.StateFlags.NORMAL, None)
        else:
            self.app_filter.modify_fg(Gtk.StateFlags.NORMAL, Color(50000, 0, 0))
    
    def on_app_filter_icon_press(self, widget, data=None, data2=None):
        self.app_filter.set_text('')
    def on_simple_rule_name_icon_press(self, widget, data=None, data2=None):
        self.simple_rule_name.set_text('')
    def on_simple_port_icon_press(self, widget, data=None, data2=None):
        self.simple_port.set_text('')
    def on_advanced_rule_name_icon_press(self, widget, data=None, data2=None):
        self.advanced_rule_name.set_text('')
    def on_advanced_to_ip_icon_press(self, widget, data=None, data2=None):
        self.advanced_to_ip.set_text('')
    def on_advanced_from_ip_icon_press(self, widget, data=None, data2=None):
        self.advanced_from_ip.set_text('')
    def on_advanced_from_port_icon_press(self, widget, data=None, data2=None):
        self.advanced_from_port.set_text('')
    def on_advanced_to_port_icon_press(self, widget, data=None, data2=None):
        self.advanced_to_port.set_text('')
    
    def on_advanced_iface_changed(self, widget, data=None, data2=None):
        # Translators: About the network interfaces in the OS
        if self.advanced_iface.get_active_text() != _("All Interfaces"):
            self.advanced_routed.set_sensitive(True)
            self.advanced_routed.set_tooltip_text(_("The IP/Port will be forward to this interface"))
        else:
            self.advanced_routed.set_sensitive(False)
            self.advanced_routed.set_tooltip_text(_("You need to set an Interface for forwarding to this another interface"))
        
        # Not allow same iface when is routed
        if self.gufw.frontend.get_policy('routed') != 'disabled':
            self.advanced_routed.remove_all()
            # Translators: About traffic
            self.advanced_routed.append_text(_("Not Forward"))
            for ifaceName in self.gufw.frontend.get_net_interfaces(self.advanced_iface.get_active_text()):
                self.advanced_routed.append_text(ifaceName)
            self.advanced_routed.set_active(0)
    
    def on_preconfig_app_changed(self, widget, data=None):
        if self.preconfig_app.get_active_text() != None:
            tooltip = self.apps.get_app(self.preconfig_app.get_active_text())[0] + '\n' + _("Ports: ") + self.apps.get_app(self.preconfig_app.get_active_text())[1]
            self.img_description.set_tooltip_text(tooltip)
            if self.apps.get_app(self.preconfig_app.get_active_text())[3]:
                self.lbl_inforule.set_text(self.apps.get_app(self.preconfig_app.get_active_text())[3])
                self.warningbox.show()
            else:
                self.warningbox.hide()
    
    def on_btnAddRuleWin_clicked(self, widget, data=None):
        if not self.gufw.frontend.get_status():
            self.gufw.show_dialog(self.win_add, _("Error: Firewall is disabled"), _("The firewall has to be enabled first"))
            return
        
        if self.tabs.get_current_page() == 0:
            self._add_rule_preconfig()
        elif self.tabs.get_current_page() == 1:
            self._add_rule_simple()
        elif self.tabs.get_current_page() == 2:
            self._add_rule_advanced()
    
    def _add(self, profile='', name='', policy='', direction='', proto='', from_ip='', from_port='', to_ip='', to_port='', insert='', iface='', routed='', logging=''):
        flag_OK = True
        flag_Warning = False
        
        # Split possible ports > 135,139,445/tcp|137,138/udp
        for from_split in from_port.split('|'):
            for to_split in to_port.split('|'):
                from_port = from_split
                to_port = to_split
                if direction == 'both':
                    cmd = self.gufw.frontend.add_rule(name, insert, policy, 'in',  iface, routed, logging, proto, from_ip, from_port, to_ip, to_port)
                    cmd = self.gufw.frontend.add_rule(name, insert, policy, 'out', iface, routed, logging, proto, from_ip, from_port, to_ip, to_port)
                else:
                    cmd = self.gufw.frontend.add_rule(name, insert, policy, direction, iface, routed, logging, proto, from_ip, from_port, to_ip, to_port)
                
                if cmd[0]:
                    flag_Warning = True
                    self.gufw.add_to_log(cmd[1])
                else:
                    flag_OK = False
                    self.gufw.add_to_log(_("Error running: ") + cmd[1] + ' > ' + cmd[2].replace('\n', ' | '))
        
        # OK
        if flag_OK and flag_Warning:
            self.gufw.set_statusbar_msg(_("Rule(s) added"))
        # Some OK, some KO
        elif flag_OK and not flag_Warning:
            self.gufw.set_statusbar_msg(_("Warning: Some rules added. Review the log"))
        # KO
        else:
            self.gufw.set_statusbar_msg(_("Error: No rules added. Review the log"))
    
    def _add_rule_preconfig(self):
        self._add(self.gufw.frontend.get_profile(),                                    # profile
                 self.preconfig_app.get_active_text(),                           # name
                 self.gufw.NUM2POLICY[self.preconfig_policy.get_active()],       # policy
                 self.gufw.NUM2DIRECTION[self.preconfig_direction.get_active()], # direction
                 '',                                                             # proto
                 '',                                                             # from IP
                 '',                                                             # from port
                 '',                                                             # to IP
                 self.apps.get_app(self.preconfig_app.get_active_text())[1])     # to port[/proto][|port[/proto]]
        self.gufw.print_rules(self.gufw.frontend.get_rules())
    
    def _add_rule_simple(self):
        if not self.simple_port.get_text():
            self.gufw.show_dialog(self.win_add, _("Insert Port"), _("You need to insert a port in the port field"))
            return
        
        if self.simple_port.get_text().upper() == 'PRISM':
            self.gufw.show_dialog(self.win_add, _("Edward Snowden's Greatest Fear"), _('"Nothing Will Change"'))
            return
        
        self._add(self.gufw.frontend.get_profile(),                                 # profile
                 self.simple_rule_name.get_text(),                            # name
                 self.gufw.NUM2POLICY[self.simple_policy.get_active()],       # policy
                 self.gufw.NUM2DIRECTION[self.simple_direction.get_active()], # direction
                 self.gufw.NUM2PROTO[self.simple_protocol.get_active()],      # protocol
                 '',                                                          # from IP
                 '',                                                          # from port
                 '',                                                          # to IP
                 self.simple_port.get_text())                                 # to port
        self.gufw.print_rules(self.gufw.frontend.get_rules())
    
    def _add_rule_advanced(self):
        # Validations
        if not self.gufw.validate_rule(self.win_add, self.advanced_from_ip.get_text(), self.advanced_from_port.get_text(), self.advanced_to_ip.get_text(), self.advanced_to_port.get_text(), self.advanced_insert.get_text(), self.advanced_routed.get_active_text()):
            return
        
        insert = self.advanced_insert.get_text()
        if insert == '0':
            insert = ''
        
        iface = ''
        # Translators: About traffic
        if self.advanced_iface.get_sensitive() and self.advanced_iface.get_active_text() != _("All Interfaces"):
            iface = self.advanced_iface.get_active_text()
        
        routed = ''
        # Translators: About traffic
        if self.advanced_routed.get_sensitive() and self.advanced_routed.get_active_text() != _("Not Forward"):
            routed = self.advanced_routed.get_active_text()
        
        to_ip = to_port = from_ip = from_port = ''
        from_ip = self.advanced_from_ip.get_text()
        from_port = self.advanced_from_port.get_text()
        to_ip = self.advanced_to_ip.get_text()
        to_port = self.advanced_to_port.get_text()
        
        self._add(self.gufw.frontend.get_profile(),                                   # profile
                 self.advanced_rule_name.get_text(),                            # name
                 self.gufw.NUM2POLICY[self.advanced_policy.get_active()],       # policy
                 self.gufw.NUM2DIRECTION[self.advanced_direction.get_active()], # direction
                 self.gufw.NUM2PROTO[self.advanced_protocol.get_active()],      # protocol
                 from_ip,                                                       # from IP
                 from_port,                                                     # from port
                 to_ip,                                                         # to IP
                 to_port,                                                       # to port
                 insert,                                                        # insert number
                 iface,                                                         # interface
                 routed,                                                        # routed
                 self.gufw.NUM2LOGGING[self.advanced_log.get_active()])         # logging        
        self.gufw.print_rules(self.gufw.frontend.get_rules())


class AppProfiles:
    """Load app profiles"""
    def __init__(self):
        self.all_apps = self._load_from_files()
        self.all_categories = self._get_all_categories()
    
    def _load_from_files(self):
        apps = {}
        os.chdir(DIR_PROFILES)
        cfg = configparser.ConfigParser()  
        cfg.read(glob.glob('*.*'))
        
        for section in cfg.sections():
            title = description = ports = categories = warning = ''
            
            if cfg.has_option(section, 'title'):
                title = cfg.get(section, 'title')
            if cfg.has_option(section, 'description'):
                description = cfg.get(section, 'description')
            if cfg.has_option(section, 'ports'):
                ports = cfg.get(section, 'ports')
            if cfg.has_option(section, 'categories'):
                categories = cfg.get(section, 'categories')
            if cfg.has_option(section, 'warning'):
                warning = cfg.get(section, 'warning')
                
            if title and description and ports and categories:
                if warning != '':
                    apps[_(title)] = [_(description), ports, _(categories), _(warning)]
                else:
                    apps[_(title)] = [_(description), ports, _(categories), '']
        return apps
    
    def _get_all_categories(self):
        all_categ = []
        for app in self.all_apps:
            categories = self.all_apps[app][2].split('|')
            for category in categories:
                if not all_categ.count(category):
                    all_categ.append(category)
        return all_categ
    
    def get_just_categories(self):
        categ = []
        for cat in self.all_categories:
            current_cat = cat.split(';')[0]
            if not categ.count(current_cat):
                categ.append(current_cat)
        categ.sort()
        # Translators: About categories
        categ.insert(0, _("All"))
        return categ
    
    def get_subcategories(self, category):
        subcateg = []
        for cat in self.all_categories:
            current_cat = cat.split(';')[0]
            # Translators: About categories
            if current_cat == category or category == _("All"):
                try:
                    current_subcat = cat.split(';')[1]
                except Exception:
                    current_subcat = ''
                if not subcateg.count(current_subcat) and current_subcat:
                    subcateg.append(current_subcat)
        subcateg.sort()
        # Translators: About subcategories
        subcateg.insert(0, _("All"))
        return subcateg
    
    def get_apps_cat_subcat(self, cat, subcat):
        apps = []
        for app in self.all_apps:
            categories = self.all_apps[app][2].split('|')
            for category in categories:
                current_cat = category.split(';')[0]
                try:
                    current_subcat = category.split(';')[1]
                except Exception:
                    current_subcat = ''
                
                # Translators: About categories
                # Translators: About subcategories
                if cat == _("All") and subcat == _("All"):
                    apps.append(app)
                # Translators: About categories
                elif cat == _("All") and subcat == current_subcat:
                    apps.append(app)
                # Translators: About categories
                elif cat == current_cat and subcat == _("All"):
                    apps.append(app)
                elif cat == current_cat and subcat == current_subcat:
                    apps.append(app)
        
        apps.sort()
        return apps
    
    def get_app(self, app):
        return self.all_apps[app]



# Gufw 20.04.1 - https://costales.github.io/projects/gufw/
# Copyright (C) 2008-2019 Marcos Alvarez Costales https://costales.github.io
#
# Gufw is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Gufw is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Gufw; if not, see http://www.gnu.org/licenses for more
# information.

import gi
import re
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk

import gettext
from gettext import gettext as _
gettext.textdomain('gufw')

NUM2UFW_LEVEL = {0 : 'off',
                 1 : 'low',
                 2 : 'medium',
                 3 : 'high',
                 4 : 'full'}
UFW_LEVEL2NUM = {'off'    : 0,
                 'low'    : 1,
                 'medium' : 2,
                 'high'   : 3,
                 'full'   : 4}

class Preferences:
    def __init__(self, gufw):
        self.gufw = gufw
        
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain('gufw')
        self.builder.add_from_file('/usr/share/gufw/ui/preferences.ui')
        
        self._set_objects_name(gufw.profile)
        self._set_initial_values()
        
        self.win_preferences.set_transient_for(gufw.winMain)
        self.builder.connect_signals(self)
        self.win_preferences.show_all()

    def _set_objects_name(self, profile_cb):
        self.win_preferences     = self.builder.get_object('preferences')
        self.ufw_logging         = self.builder.get_object('ufw_logging')
        self.gufw_logging        = self.builder.get_object('gufw_logging')
        self.gufw_confirm_delete = self.builder.get_object('gufw_confirm_delete')
        self.report_interval     = self.builder.get_object('report_interval')
        self.list_profiles       = self.builder.get_object('profiles_list')
        self.list_selection      = self.builder.get_object('profiles_selection')
    
    def _set_initial_values(self):
        self.ufw_logging.set_active(UFW_LEVEL2NUM[self.gufw.frontend.get_ufw_logging()])
        if self.gufw.frontend.get_logging():
            self.gufw_logging.set_active(True)
        if self.gufw.frontend.get_config_value('ConfirmDeteleRule') == 'yes':
            self.gufw_confirm_delete.set_active(True)
        
        if self.gufw.frontend.get_config_value('RefreshInterval'):
            self.report_interval.set_value(int(self.gufw.frontend.get_config_value('RefreshInterval')))
        else:
            self.report_interval.set_value(3)
        
        # Profiles (model)
        self.profile_rows = Gtk.ListStore(str)
        for profile in self.gufw.frontend.get_all_profiles():
            self.profile_rows.append([profile])
        self.list_profiles.set_model(self.profile_rows)
        # Profiles (Editable one column)
        renderer_editable_profile = Gtk.CellRendererText()
        renderer_editable_profile.set_property('editable', True)
        type_profile_column = Gtk.TreeViewColumn(_("Profile"), renderer_editable_profile, text=0)
        self.list_profiles.append_column(type_profile_column)
        # Profiles (Edit row event)
        renderer_editable_profile.connect('edited', self._rename_profile)
    
    def _rename_profile(self, widget, path, new_name):
        """2 Click on profile"""
        # None is for internal use
        if new_name == 'None':
            self.gufw.show_dialog(self.win_preferences, _("Profile not valid"), _("You can't use this profile name"))
            return
        # Not empty
        if not new_name:
            self.gufw.show_dialog(self.win_preferences, _("Profile not valid"), _("Enter at least one character"))
            return
        # Length
        if len(new_name) > 15:
            self.gufw.show_dialog(self.win_preferences, _("Profile not valid"), _("Too long! (max. 15 characters)"))
            return
        # Check only ASCII characters and no spaces
        if not re.match('^[A-Za-z0-9_-]*$', new_name):
            self.gufw.show_dialog(self.win_preferences, _("Profile not valid"), _("Use only letters, numbers, dashes and underscores"))
            return
        # Exist?
        for searched_profile in self.profile_rows:
            if searched_profile[0] == new_name:
                self.gufw.show_dialog(self.win_preferences, _("Profile exist"), _("There is a profile with the same name"))
                return
        # Not the default
        previous_name = self.profile_rows[path][0]
        if previous_name == self.gufw.frontend.get_profile():
            self.gufw.show_dialog(self.win_preferences, _("Current profile"), _("You can't rename the current profile"))
            return
        
        self.gufw.frontend.rename_profile(self.profile_rows[path][0], new_name)
        self.profile_rows[path][0] = new_name
        self.gufw.profile.remove(int(path))
        self.gufw.profile.insert_text(int(path), new_name)
        self.gufw.add_to_log(_("Edited Profile: ") + new_name)
    
    def on_AddProfile_btn_clicked(self, widget, data=None):
        ind = 0
        exist_profile = True
        while exist_profile:
            ind += 1
            new_name = _("Profile") + str(ind)
            exist_profile = False
            for searched_profile in self.profile_rows:
                if searched_profile[0] == new_name:
                    exist_profile = True
                    break # Next ind
        
        self.gufw.frontend.add_profile(new_name)
        ind = self.gufw.frontend.get_all_profiles().index(new_name)
        self.profile_rows.append([new_name])
        self.gufw.profile.append_text(new_name)
        self.gufw.add_to_log(_("Created Profile: ") + new_name)
        
    def on_DeleteProfile_btn_clicked(self, widget, data=None):
        model, treeiter = self.list_selection.get_selected()
        if treeiter == None:
            self.gufw.show_dialog(self.win_preferences, _("Select a profile"), _("You need to select a profile for deleting"))
        else:
            deleted_profile = model.get_value(treeiter, 0)
            if deleted_profile == self.gufw.frontend.get_profile():
                self.gufw.show_dialog(self.win_preferences, _("Profile not erasable"), _("You can't remove the current profile"))
            else:
                self.gufw.profile.remove(self.gufw.frontend.get_all_profiles().index(deleted_profile))
                self.gufw.frontend.delete_profile(deleted_profile)
                self.profile_rows.remove(treeiter)
                self.gufw.add_to_log(_("Deleted Profile: ") + deleted_profile)
    
    def on_ufw_logging_changed(self, widget, data=None):
        self.gufw.frontend.set_ufw_logging(NUM2UFW_LEVEL[self.ufw_logging.get_active()])
        self.gufw.add_to_log(_("ufw Logging: ") + self.ufw_logging.get_active_text())
    
    def on_gufw_logging_toggled(self, widget, data=None):
        if self.gufw_logging.get_active():
            self.gufw.frontend.set_logging(True)
            self.gufw.add_to_log(_("Gufw Logging: Enabled"))
        else:
            self.gufw.add_to_log(_("Gufw Logging: Disabled"))
            self.gufw.frontend.set_logging(False)
    
    def on_gufw_confirm_delete_toggled(self, widget, data=None):
        if self.gufw_confirm_delete.get_active():
            self.gufw.frontend.set_config_value('ConfirmDeteleRule', 'yes')
            self.gufw.add_to_log(_("Confirm Delete Dialog: Enabled"))
        else:
            self.gufw.frontend.set_config_value('ConfirmDeteleRule', 'no')
            self.gufw.add_to_log(_("Confirm Delete Dialog: Disabled"))
    
    def on_close_btn_clicked(self, widget, data=None):
        self.win_preferences.destroy()
    
    def on_preferences_delete_event(self, widget, data=None):
        self.win_preferences.destroy()
    
    def on_report_interval_scale_button_release_event(self, widget, data=None):
        refresh_time = int(self.report_interval.get_value())
        self.gufw.frontend.set_config_value('RefreshInterval', refresh_time)
        # Translators: Refresh the live traffic on GUI
        self.gufw.add_to_log(_("Refresh Interval: ") + str(refresh_time) + '"')



# Gufw 20.04.1 - https://costales.github.io/projects/gufw/
# Copyright (C) 2008-2019 Marcos Alvarez Costales https://costales.github.io
#
# Gufw is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Gufw is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Gufw; if not, see http://www.gnu.org/licenses for more
# information.

from gufw.model.firewall import Firewall


class Frontend():
    def __init__(self):
        self.firewall = Firewall()
    
    
    # PROFILE
    def get_profile(self):
        return self.firewall.get_profile()
    
    def set_profile(self, profile):
        return self.firewall.set_profile(profile)
    
    def get_all_profiles(self):
        return self.firewall.get_all_profiles()
    
    def add_profile(self, profile):
        self.firewall.add_profile(profile)
    
    def delete_profile(self, profile):
        self.firewall.delete_profile(profile)
    
    def rename_profile(self, old_name, new_name):
        self.firewall.rename_profile(old_name, new_name)
    
    def import_profile(self, profile_file):
        self.firewall.import_profile(profile_file)
    
    def export_profile(self, profile_file):
        self.firewall.export_profile(profile_file)
    
    # FIREWALL > For current profile!
    def get_status(self):
        return self.firewall.get_status()
    
    def set_status(self, status):
        self.firewall.set_status(status)
    
    def get_policy(self, policy):
        return self.firewall.get_policy(policy)
    
    def set_policy(self, policy, value):
        self.firewall.set_policy(policy, value)
    
    def reset(self):
        self.firewall.reset()
    
    
    # RULES > For current profile!
    def get_rules(self, force_fw_on=True):
        return self.firewall.get_rules(force_fw_on)
    
    def get_number_rules(self):
        return self.firewall.get_number_rules()
    
    def add_rule(self, description, insert='', policy='', direction='', iface='', routed='', logging='', proto='', from_ip='', from_port='', to_ip='', to_port=''):
        return self.firewall.add_rule(description, insert, policy, direction, iface, routed, logging, proto, from_ip, from_port, to_ip, to_port)
    
    def delete_rule(self, num):
        return self.firewall.delete_rule(num)
    
    
    
    # LOGGING
    # ufw LOGGING
    def get_ufw_logging(self):
        return self.firewall.get_ufw_logging()
    
    def set_ufw_logging(self, level):
        self.firewall.set_ufw_logging(level)
    
    # Gufw LOGGING
    def get_logging(self):
        return self.firewall.get_logging()
    
    def set_logging(self, status):
        self.firewall.set_logging(status)
    
    def get_log(self):
        return self.firewall.get_log()
    
    def add_to_log(self, msg):
        return self.firewall.add_to_log(msg)
    
    def refresh_log(self):
        self.firewall.refresh_log()
    
    
    
    # LISTENING REPORT
    def get_listening_report(self):
        return self.firewall.get_listening_report()
    
    
    
    # GUI needs
    def get_config_value(self, attrib):
        return self.firewall.get_cfg_value(attrib)
    
    def set_config_value(self, attrib, value):
        self.firewall.set_cfg_value(attrib, value)
    
    def get_internal_ip(self):
        return self.firewall.get_internal_ip()
    
    def get_net_interfaces(self, exclude_iface=''):
        return self.firewall.get_net_interfaces(exclude_iface)



# Gufw 20.04.1 - https://costales.github.io/projects/gufw/
# Copyright (C) 2008-2019 Marcos Alvarez Costales https://costales.github.io
#
# Gufw is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Gufw is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Gufw; if not, see http://www.gnu.org/licenses for more
# information.

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk


class About:
    def __init__(self, gufw):
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain('gufw')
        self.builder.add_from_file('/usr/share/gufw/ui/about.ui')
        
        self.win_about = self.builder.get_object('about')
        self.win_about.set_transient_for(gufw.winMain)
        self.win_about.connect('response', lambda d, r: d.destroy())
        self.win_about.show()




# Gufw 20.04.1 - https://costales.github.io/projects/gufw/
# Copyright (C) 2008-2019 Marcos Alvarez Costales https://costales.github.io
#
# Gufw is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Gufw is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Gufw; if not, see http://www.gnu.org/licenses for more
# information.

import time, os, shutil, subprocess, configparser


class Backend():
    UFW_PATH    = '/usr/sbin/ufw'
    UFW_DEFAULT = '/etc/default/ufw'
    UFW_CONF    = '/etc/ufw/ufw.conf'
    UFW_SYSCTL  = '/etc/ufw/sysctl.conf'
    GUFW_PATH   = '/etc/gufw'
    GUFW_CFG    = '/etc/gufw/gufw.cfg'
    GUFW_LOG    = '/var/log/gufw.log'
    
    def __init__(self):
        pass
    
    def _run_cmd(self, cmd, lang_c=False):
        if lang_c:
            proc = subprocess.Popen(cmd, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env={'LANG':'C'})
        else:
            proc = subprocess.Popen(cmd, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        stdout,stderr=proc.communicate()
        
        if stderr: # Error
            while stderr[-1:] == '\n':
                stderr = stderr[:-1]
            return stderr.decode('utf-8')
        else: # OK
            while stdout[-1:] == '\n':
                stdout = stdout[:-1]
            return stdout.decode('utf-8')
    
    def get_status(self):
        if 'Status: active' in self._run_cmd([self.UFW_PATH, 'status'], True):
            return True
        else:
            return False
    
    def get_policy(self, policy):
        if policy == 'incoming':
            ufw_default_incoming = self._run_cmd(['grep', 'DEFAULT_INPUT_POLICY', self.UFW_DEFAULT])
            if 'ACCEPT' in ufw_default_incoming:
                return 'allow'
            elif 'DROP' in ufw_default_incoming:
                return 'deny'
            elif 'REJECT' in ufw_default_incoming:
                return 'reject'
        
        elif policy == 'outgoing':
            ufw_default_outgoing = self._run_cmd(['grep', 'DEFAULT_OUTPUT_POLICY', self.UFW_DEFAULT])
            if 'ACCEPT' in ufw_default_outgoing:
                return 'allow'
            elif 'DROP' in ufw_default_outgoing:
                return 'deny'
            elif 'REJECT' in ufw_default_outgoing:
                return 'reject'
        
        elif policy == 'routed':
            ufw_default_outgoing = self._run_cmd(['grep', 'net/ipv4/ip_forward=', self.UFW_SYSCTL])
            routed_status = ufw_default_outgoing.replace(" ", "")
            if "#net/ipv4/ip_forward=1" in routed_status:
                return 'disabled'
            
            ufw_default_outgoing = self._run_cmd(['grep', 'DEFAULT_FORWARD_POLICY', self.UFW_DEFAULT])
            if 'ACCEPT' in ufw_default_outgoing:
                return 'allow'
            elif 'DROP' in ufw_default_outgoing:
                return 'deny'
            elif 'REJECT' in ufw_default_outgoing:
                return 'reject'
    
    def get_ufw_logging(self):
        ufw_cmd = self._run_cmd(['cat', self.UFW_CONF])
        if 'LOGLEVEL=full' in ufw_cmd:
            return 'full'
        elif 'LOGLEVEL=high' in ufw_cmd:
            return 'high'
        elif 'LOGLEVEL=medium' in ufw_cmd:
            return 'medium'
        elif 'LOGLEVEL=low' in ufw_cmd:
            return 'low'
        else:
            return 'off'
    
    def set_status(self, status):
        if not status:
            cmd = [self.UFW_PATH, 'disable']
        else:
            cmd = [self.UFW_PATH, '--force', 'enable']
        
        self._run_cmd(cmd)
    
    def set_policy(self, policy, value):
        if policy == 'incoming':
            if value == 'allow':
                cmd = [self.UFW_PATH, 'default', 'allow', 'incoming']
            elif value == 'deny':
                cmd = [self.UFW_PATH, 'default', 'deny', 'incoming']
            elif value == 'reject':
                cmd = [self.UFW_PATH, 'default', 'reject', 'incoming']
        
        elif policy == 'outgoing':
            if value == 'allow':
                cmd = [self.UFW_PATH, 'default', 'allow', 'outgoing']
            elif value == 'deny':
                cmd = [self.UFW_PATH, 'default', 'deny', 'outgoing']
            elif value == 'reject':
                cmd = [self.UFW_PATH, 'default', 'reject', 'outgoing']
        
        elif policy == 'routed':
            if value == 'allow':
                cmd = [self.UFW_PATH, 'default', 'allow', 'routed']
            elif value == 'deny':
                cmd = [self.UFW_PATH, 'default', 'deny', 'routed']
            elif value == 'reject':
                cmd = [self.UFW_PATH, 'default', 'reject', 'routed']
        
        if cmd:
            self._run_cmd(cmd)
    
    def set_ufw_logging(self, logging):
        if logging == 'off':
            cmd = [self.UFW_PATH, 'logging', 'off']
        elif logging == 'low':
            cmd = [self.UFW_PATH, 'logging', 'low']
        elif logging == 'medium':
            cmd = [self.UFW_PATH, 'logging', 'medium']
        elif logging == 'high':
            cmd = [self.UFW_PATH, 'logging', 'high']
        elif logging == 'full':
            cmd = [self.UFW_PATH, 'logging', 'full']
        
        if cmd:
            self._run_cmd(cmd)
    
    def reset_fw(self):
        cmd = [self.UFW_PATH, '--force', 'reset']
        self._run_cmd(cmd, True)
    
    def get_cfg_value(self, attribute):
        cfg = configparser.ConfigParser()
        try:
            if not cfg.read([self.GUFW_CFG]):
                return ''
        except Exception:
            os.remove(self.GUFW_CFG)
            return ''
        
        if not cfg.has_option('GufwConfiguration', attribute):
            return ''
            
        return cfg.get('GufwConfiguration', attribute)
    
    def set_cfg_value(self, attr, value):
        cfg_file = self.GUFW_CFG
        if not os.path.isfile(cfg_file):
            f = open(cfg_file, 'w')
            cfg = configparser.ConfigParser()
            cfg.add_section("GufwConfiguration")
            cfg.write(f)  
            f.close()  
            os.chmod(cfg_file, 0o600) # Just an admin can read this file
        
        cfg = configparser.ConfigParser()
        cfg.read(cfg_file)
        cfg.set('GufwConfiguration', attr, str(value))
        f = open(cfg_file, 'w')
        cfg.write(f)  
        f.close()
    
    def set_profile_values(self, profile, status, incoming, outgoing, routed, rules):
        if not os.path.exists(self.GUFW_PATH):
            os.makedirs(self.GUFW_PATH)
        
        file_name = profile + '.profile'
        file_path = os.path.join(self.GUFW_PATH, file_name)
        
        cfg = configparser.ConfigParser()
        
        cfg.add_section("fwBasic")
        if status:
            cfg.set('fwBasic', 'status', 'enabled')
        else:
            cfg.set('fwBasic', 'status', 'disabled')
        cfg.set('fwBasic', 'incoming', incoming)
        cfg.set('fwBasic', 'outgoing', outgoing)
        cfg.set('fwBasic', 'routed',   routed)
        
        i = 0
        for rule in rules:
            if not rule['command']:
                continue
            section = 'Rule' + str(i)
            cfg.add_section(section)
            cfg.set(section, 'ufw_rule',    rule['ufw_rule'])
            cfg.set(section, 'description', rule['description'])
            cfg.set(section, 'command',     rule['command'])
            cfg.set(section, 'policy',      rule['policy'])
            cfg.set(section, 'direction',   rule['direction']) 
            cfg.set(section, 'protocol',    rule['protocol']) 
            cfg.set(section, 'from_ip',     rule['from_ip']) 
            cfg.set(section, 'from_port',   rule['from_port']) 
            cfg.set(section, 'to_ip',       rule['to_ip']) 
            cfg.set(section, 'to_port',     rule['to_port']) 
            cfg.set(section, 'iface',       rule['iface']) 
            cfg.set(section, 'routed',      rule['routed']) 
            cfg.set(section, 'logging',     rule['logging'] )
            i += 1
        
        f = open(file_path, 'w')
        cfg.write(f)  
        f.close()
        os.chmod(file_path, 0o600) # Just an admin can read this file
    
    def get_profile_values(self, profile):
        status = False
        incoming = ''
        outgoing = ''
        routed = ''
        rules = []
        
        if profile[:1] == '/':
            file_path = profile
        else:
            file_name = profile + '.profile'
            file_path = os.path.join('/etc', 'gufw', file_name)
        
        cfg = configparser.ConfigParser()
        if not cfg.read([file_path]):
            return (status, incoming, outgoing, routed, rules)
        
        if not cfg.has_section('fwBasic'):
            return (status, incoming, outgoing, routed, rules)
        
        for section in cfg.sections():
            
            if section == 'fwBasic':
                if cfg.get('fwBasic', 'status') == 'disabled':
                    status = False
                else:
                    status = True
                incoming = cfg.get('fwBasic', 'incoming')
                outgoing = cfg.get('fwBasic', 'outgoing')
                # Previous Gufw profiles
                try:
                    routed = cfg.get('fwBasic', 'routed')
                except configparser.NoOptionError:
                    routed = self.get_policy('routed')
                
            
            else:
                rule = {'ufw_rule'   : cfg.get(section, 'ufw_rule'), 
                        'description': cfg.get(section, 'description'),
                        'command'    : cfg.get(section, 'command'),
                        'policy'     : cfg.get(section, 'policy'),
                        'direction'  : cfg.get(section, 'direction'),
                        'protocol'   : cfg.get(section, 'protocol'),
                        'from_ip'    : cfg.get(section, 'from_ip'),
                        'from_port'  : cfg.get(section, 'from_port'),
                        'to_ip'      : cfg.get(section, 'to_ip'),
                        'to_port'    : cfg.get(section, 'to_port'),
                        'iface'      : cfg.get(section, 'iface'),
                        'logging'    : cfg.get(section, 'logging')}
                # Previous Gufw profiles
                try:
                    rule['routed'] = cfg.get(section, 'routed')
                except configparser.NoOptionError:
                    rule['routed'] = ''
                rules.append(rule)
        
        return (status, incoming, outgoing, routed, rules)
    
    
    def delete_file_profile(self, profile):
        dst = os.path.join(self.GUFW_PATH, profile + '.profile')
        try:
            os.remove(dst)
        except Exception:
            pass
    
    def rename_file_profile(self, old, new):
        src = os.path.join(self.GUFW_PATH, old + '.profile')
        dst = os.path.join(self.GUFW_PATH, new + '.profile')
        
        if os.path.isfile(dst): # User could import a profile with same English name > It's OK
            return
        print('renaming '+src+' a '+dst)
        try:
            os.rename(src, dst)
        except Exception:
            pass
    
    def export_profile(self, profile, dst):
        src = os.path.join(self.GUFW_PATH, profile + '.profile')
        try:
            shutil.copyfile(src, dst)
            os.chmod(dst, 0o600) # Avoid other users change a profile
        except Exception:
            pass
    
    def refresh_log(self):
        f = open(self.GUFW_LOG, 'w')
        f.close()
    
    def add_to_log(self, msg):
        try:
            f = open(self.GUFW_LOG, 'r')
            log = f.readlines()
            f.close()
        except Exception:
            log = ''
        
        new_line = '[' + time.strftime('%x %X') + '] ' + msg
        if log:
            new_line = new_line + '\n'
        
        f = open(self.GUFW_LOG, 'w')
        f.write(new_line)
        f.writelines(log)
        f.close()
        
        return new_line

    def get_log(self):
        if not os.path.isfile(self.GUFW_LOG):
            return ''
        
        cmd = self._run_cmd(['cat', self.GUFW_LOG])
        if not cmd:
            return ''
        
        return cmd
    
    def get_rules(self):
        rules = self._run_cmd([self.UFW_PATH, 'status', 'numbered'], True)
        lines = rules.split('\n')
        return_rules = []
        
        for line in lines:
            if line and 'ALLOW' in line or 'DENY' in line or 'LIMIT' in line or 'REJECT' in line:
                rule = line.split('] ')
                return_rules.append(' '.join(rule[1].split()))
        
        return return_rules
    
    def get_number_rules(self):
        numb = 0
        rules = self._run_cmd([self.UFW_PATH, 'status', 'numbered'], True)
        lines = rules.split('\n')
        return_rules = []
        
        for line in lines:
            if line and 'ALLOW' in line or 'DENY' in line or 'LIMIT' in line or 'REJECT' in line:
                numb = numb + 1
        
        return numb
    
    def add_rule(self, insert, policy, direction, iface, routed, logging, proto, from_ip, from_port, to_ip, to_port):
        # ufw [route] [insert NUM] allow|deny|reject|limit [in|out on INTERFACE] [log|log-all] [proto protocol] [from ADDRESS [port PORT]] [to ADDRESS [port PORT]]
        cmd_rule = [self.UFW_PATH]
        
        # route
        if routed:
            cmd_rule.append('route')
        
        # Insert Number
        if insert:
            cmd_rule.extend(['insert', str(int(insert))])
        
        # Policy
        cmd_rule.append(policy)
        
        # Direction
        cmd_rule.append(direction)
        
        # Interface
        if iface:
            cmd_rule.extend(['on', iface])
        
        # Routed on
        if routed:
            if direction == 'in':
                cmd_rule.extend(['out', 'on', routed])
            else:
                cmd_rule.extend(['in', 'on', routed])
        
        # Logging
        if logging:
            cmd_rule.append(logging)
        
        # Proto
        if '/tcp' in from_port or '/tcp' in to_port:
            cmd_rule.extend(['proto', 'tcp'])
        elif '/udp' in from_port or '/udp' in to_port:
            cmd_rule.extend(['proto', 'udp'])
        elif proto:
            cmd_rule.extend(['proto', proto])
        
        # From IP
        if not from_ip:
            cmd_rule.extend(['from', 'any'])
        else:
            cmd_rule.extend(['from', from_ip])
        # From Port
        if from_port:
            if '/tcp' in from_port:
                from_port = from_port.replace('/tcp', '')
            if '/udp' in from_port:
                from_port = from_port.replace('/udp', '')
            cmd_rule.extend(['port', from_port])
        
        # To IP
        if not to_ip:
            cmd_rule.extend(['to', 'any'])
        else:
            cmd_rule.extend(['to', to_ip])
        # To Port
        if to_port:
            if '/tcp' in to_port:
                to_port = to_port.replace('/tcp', '')
            if '/udp' in to_port:
                to_port = to_port.replace('/udp', '')
            cmd_rule.extend(['port', to_port])
        
        # Launch
        cmd = self._run_cmd(cmd_rule, True)
        
        result = []
        result.append(' '.join(cmd_rule))
        result.append(cmd)
        
        return result # cmd | ufw result
    
    def delete_rule(self, num):
        delete_rule = [self.UFW_PATH, '--force', 'delete', str(num)]
        cmd = self._run_cmd(delete_rule)
        
        result = []
        result.append(' '.join(delete_rule))
        result.append(cmd)
        
        return result # cmd | ufw result
    
    def get_net_interfaces(self):
        cmd_ifaces = ['ls', '/sys/class/net']
        cmd = self._run_cmd(cmd_ifaces)
        ifaces = cmd.split('\n')
        
        result = []
        for iface in ifaces:
            if iface:
                result.append(iface)
        
        return result
    
    def get_net_ip(self):
        cmd_ip = ['hostname', '--all-ip-addresses']
        cmd = self._run_cmd(cmd_ip)
        ips = cmd.split('\n')
        
        if len(ips) > 0:
            return ips[0].strip()
        else:
            return '127.0.0.1'
    
    def get_listening_report(self):
        return_report = []
        actual_protocol = 'None'
        ufw_report = self._run_cmd([self.UFW_PATH, 'show', 'listening'], True)
        report_lines = ufw_report.replace('\n   [', '%')
        report_lines = report_lines.split('\n')
        
        for descomponent_report in report_lines:
            # Set actual protocol
            if not descomponent_report:
                continue
            if 'tcp6:' in descomponent_report:
                actual_protocol = 'TCP6'
                continue
            if 'tcp:' in descomponent_report:
                actual_protocol = 'TCP'
                continue
            if 'udp6:' in descomponent_report:
                actual_protocol = 'UDP6'
                continue
            if 'udp:' in descomponent_report:
                actual_protocol = 'UDP'
                continue
            
            policy = 'None'
            descomponent_report = descomponent_report.strip()
            descomponent_report = descomponent_report.replace('(', '')
            descomponent_report = descomponent_report.replace(')', '')
            
            if ']' in descomponent_report:
                descomponent_policy = descomponent_report.split(']')
                if 'allow' in descomponent_policy[1]:
                    policy = 'allow'
                elif 'deny' in descomponent_policy[1]:
                    policy = 'deny'
                elif 'reject' in descomponent_policy[1]:
                    policy = 'reject'
                elif 'limit' in descomponent_policy[1]:
                    policy = 'limit'
            
            descomponent_report = descomponent_report.split('%')
            descomponent_fields = descomponent_report[0].split(' ')
            # Order: protocol % port % address % application % policy
            return_report.append('%'.join([actual_protocol, descomponent_fields[0], descomponent_fields[1], descomponent_fields[2], policy]))
        
        return return_report



# Gufw 20.04.1 - https://costales.github.io/projects/gufw/
# Copyright (C) 2008-2019 Marcos Alvarez Costales https://costales.github.io
#
# Gufw is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Gufw is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Gufw; if not, see http://www.gnu.org/licenses for more
# information.

import os, sys
from stat import *

class Instance:
    def __init__(self):
        self.pid_file = '/tmp/gufw.pid'
        self._check_is_root()
        self._check_dir_writable('/etc')
        self._check_dir_writable('/lib')
        self._check_instance()
        self._start_application()
    
    def _check_is_root(self):
        if os.geteuid() != 0:
            from gi.repository import Gtk
            dialog = Gtk.MessageDialog(None, 0, Gtk.MessageType.INFO, Gtk.ButtonsType.OK, "Run as superuser")
            dialog.format_secondary_text("Just run this command in the shell: gufw  or  sudo gufw")
            dialog.run()
            dialog.destroy()
            exit(0)
    
    def _check_dir_writable(self, directory):
        if bin(os.stat(directory)[ST_MODE])[-5] == '0' and bin(os.stat(directory)[ST_MODE])[-2] == '0': # get chmod /etc
            return
        
        from gi.repository import Gtk
        import gettext
        from gettext import gettext as _
        gettext.textdomain('gufw')
        
        dialog = Gtk.MessageDialog(None, 0, Gtk.MessageType.INFO, Gtk.ButtonsType.OK, _("Error: %s is writable" % directory))
        dialog.format_secondary_text(_("Your %s directory is writable.\nFix it running from Terminal:" % directory) + "\n\nsudo chmod 755 " + directory)
        dialog.run()
        dialog.destroy()
        exit(0)

    def _check_instance(self):
        if not os.path.isfile(self.pid_file):
            return
        
        # Read the pid from file
        pid = 0
        try:
            pid_file = open(self.pid_file, 'rt')
            data = pid_file.read()
            pid_file.close()
            pid = int(data)
        except Exception:
            pass
        
        # Check whether the process specified exists
        if pid == 0:
            return
        try:
            os.kill(pid, 0) # exception if the pid is invalid
        except Exception:
            return
        
        from gi.repository import Gtk
        import gettext
        from gettext import gettext as _
        gettext.textdomain('gufw')
        
        dialog = Gtk.MessageDialog(None, 0, Gtk.MessageType.INFO, Gtk.ButtonsType.OK, _("Please, just one Gufw's instance"))
        dialog.format_secondary_text(_("Gufw is already running. If this is wrong, remove the file: ") + self.pid_file)
        dialog.run()
        dialog.destroy()
        exit(0)
    
    def _start_application(self):
        if self._under_ssh():
            return
        
        pid_file = open(self.pid_file, 'wt')
        pid_file.write(str(os.getpid()))
        pid_file.close()
    
    def _under_ssh(self):
        try:
            if sys.argv[1] == '-ssh':
                return True
        except Exception:
            pass
        
        return False
    
    def exit_app(self):
        try:
            os.remove(self.pid_file)
        except Exception:
            pass



#!/usr/bin/env python3

# Gufw 20.04.1 - https://costales.github.io/projects/gufw/
# Copyright (C) 2008-2019 Marcos Alvarez Costales https://costales.github.io
#
# Gufw is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
# 
# Gufw is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Gufw; if not, see http://www.gnu.org/licenses for more
# information.

from DistUtilsExtra.auto import setup
import glob

# Create data files
data = [ ('/usr/share/polkit-1/actions/', ['policykit/actions/com.ubuntu.pkexec.gufw.policy']),
         ('/etc/gufw/app_profiles',       glob.glob("data/app_profiles/*.*")) ]

# Setup stage
setup(
    name         = "gufw",
    version      = "20.04.1",
    description  = "An easy, intuitive, way to manage your Linux firewall. It supports common tasks such as allowing or blocking pre-configured, common p2p, or individual ports port(s), and many others!",
    author       = "Marcos Alvarez Costales https://costales.github.io",
    author_email = "https://costales.github.io",
    url          = "https://costales.github.io/projects/gufw/",
    license      = "GPL3",
    data_files   = data
    )



# Gufw 20.04.1 - https://costales.github.io/projects/gufw/
# Copyright (C) 2008-2019 Marcos Alvarez Costales https://costales.github.io
#
# Gufw is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Gufw is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Gufw; if not, see http://www.gnu.org/licenses for more
# information.

import gi
import re
gi.require_version('Gtk', '3.0')
from gi.repository import GLib, Gtk


REFRESH_TIME = 3 # Default interval refresh


class ListeningReport():
    
    def __init__(self, gufw):
        self.gufw = gufw
        self.previous_report = []
        self.running_listening = True
        self.paused_listening = False
        
        self._show_report()
        
        if self.gufw.frontend.get_config_value('RefreshInterval'):
            time = int(self.gufw.frontend.get_config_value('RefreshInterval'))
        else:
            time = REFRESH_TIME
        GLib.timeout_add((time * 1000), self._show_report)
        
    def _show_report(self):
        if not self.running_listening:
            return False
        
        report = self.gufw.frontend.get_listening_report()
        self._view_report(report, self.previous_report)
        self.previous_report = report
        
        while Gtk.events_pending(): # Not freeze main win
            Gtk.main_iteration()
        
        return True
    
    def stopping(self):
        self.running_listening = False
    
    def set_pause(self, value):
        self.paused_listening = value
    
    def _view_report(self, report, previous_report):
        if self.paused_listening:
            return
        # Selected?
        protocol = port = address = app = ''
        (model, rows) = self.gufw.tv_report.get_selection().get_selected_rows()
        if len(rows) == 1:
            iter_row = self.gufw.listening_model.get_iter(rows[0],)
            protocol = self.gufw.listening_model.get_value(iter_row, 0)
            port     = str(self.gufw.listening_model.get_value(iter_row, 1))
            address  = self.gufw.listening_model.get_value(iter_row, 2)
            app      = self.gufw.listening_model.get_value(iter_row, 3)
        
        row = 0
        self.gufw.listening_model.clear()
        
        for line in report:
            line_split = line.split('%')
            row += 1
            iter_row = self.gufw.listening_model.insert(row)
            self.gufw.listening_model.set_value(iter_row, 0, line_split[0].strip())      # protocol
            self.gufw.listening_model.set_value(iter_row, 1, int(line_split[1].strip())) # port
            self.gufw.listening_model.set_value(iter_row, 2, line_split[2].strip())      # address
            self.gufw.listening_model.set_value(iter_row, 3, line_split[3].strip())      # app
            self.gufw.listening_model.set_value(iter_row, 5, row)                        # number
            
            if self.gufw.frontend.get_status():
                if line_split[4] == 'allow':
                    self.gufw.listening_model.set_value(iter_row, 4, self.gufw.RED)
                elif line_split[4] == 'deny':
                    self.gufw.listening_model.set_value(iter_row, 4, self.gufw.GREEN)
                elif line_split[4] == 'reject':
                    self.gufw.listening_model.set_value(iter_row, 4, self.gufw.BLUE)
                elif line_split[4] == 'limit':
                    self.gufw.listening_model.set_value(iter_row, 4, self.gufw.ORANGE)
        
        # Previously selected? > Get current row because treeview could be sorted
        if protocol and port:
            select_this_row = -1
            i = 0
            while True:
                try:
                    iter_row = self.gufw.listening_model.get_iter(i,)
                    if (protocol == self.gufw.listening_model.get_value(iter_row, 0) and
                        port     == str(self.gufw.listening_model.get_value(iter_row, 1)) and
                        address  == self.gufw.listening_model.get_value(iter_row, 2) and
                        app      == self.gufw.listening_model.get_value(iter_row, 3)):
                        select_this_row = i
                except Exception:
                    break
                i += 1
            
            if select_this_row != -1:
                self.gufw.tv_report.set_cursor(select_this_row)
